VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cDiskInfo"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
' ***************************************************************************
' Several of these routines were found at a site that is
' maintained by Randy Birch.
' http://vbnet.mvps.org/index.html
'
' Runs only on Windows NT and newer, for two reasons:
'
' The standard device input/output control codes are available only
' on Windows NT and newer.
'
' On Windows NT based operating systems, an application can use the
' DeviceIoControl API function to perform direct input and output
' operations on, or retrieve information about, a floppy disk drive,
' hard disk drive, tape drive, or CD-ROM drive. This page demonstrates
' how to retrieveinformation about the installed  physical drives on
' the system. It uses the CreateFile function to obtain a device handle
' to the physical drives, and then uses DeviceIoControl with the
' IOCTL_DISK_GET_DRIVE_GEOMETRY control code to fill a DISK_GEOMETRY
' structure with information about the drive.
'
' The IOCTL_DISK_GET_DRIVE_GEOMETRY control code returns information
' about the physical disk's geometry: type, number of cylinders,
' tracks per cylinder, sectors per track, and bytes per sector. The
' DISK_GEOMETRY structure describes the geometry of disk devices and
' media. Note that the first parameter of the DISK_GEOMETRY type is
' defined as a LARGE_INTEGER, requiring the VB Currency data type to
' be used instead of a traditional Long.
'
' ===========================================================================
'    DATE      NAME / DESCRIPTION
' -----------  --------------------------------------------------------------
' 03-MAR-2000  Kenneth Ives  kenaso@tx.rr.com
'              Wrote module
' 02-Nov-2009  Kenneth Ives  kenaso@tx.rr.com
'              Replaced FileExists() and PathExists() routines with
'              IsPathValid() routine.
' 15-May-2010  Kenneth Ives  kenaso@tx.rr.com
'              - Added 3 holding areas to type structure DRIVE_INFORMATION
'              - Updated SwapBytes(), GetMfrInfo() and GetDriveInfo() routines.
'              - Corrected formatting of volume serial number in
'                GetVolumeInfo() routine
'              - Added minor documentation
' 21-Jul-2010  Kenneth Ives  kenaso@tx.rr.com
'              Added GetDriveLetters() routine
' 15-Dec-2010  Kenneth Ives  kenaso@tx.rr.com
'              Rearranged parameters and made return parameters optional in
'              GetMfrInfo() routine.
' 10-Jul-2011  Kenneth Ives  kenaso@tx.rr.com
'              - Replaced type structure DISK_INFORMATION with multiple
'                properties so user can access disk information more easily.
'              - Updated GetDriveLetters(), GetClusterSize(), DisplayNumber()
'                routines.
'              - Added GetDiskNumber() routine.
'              - Removed some obsolete code.
' 14-Jul-2011  Kenneth Ives  kenaso@tx.rr.com
'              - Rewrote GetDriveInfo(), GetClusterSize() routines.
'              - Updated SpecificTypeOfDrive() to catch USB devices.
'              - Updated GetDiskNumber() routine to capture USB information.
'              - Renamed GetMediaSize() routine to GetDriveTypeExtra().
'              - Updated GetMediaInfo() routine. Moved from Public to Private.
'              - Rewrote GetDiskGeometryEx() routine.
' 20-Jul-2011  Kenneth Ives  kenaso@tx.rr.com
'              - Updated GetMfrInfo(), GetPhysicalDriveInfo() routines.
' 20-Nov-2011  Kenneth Ives  kenaso@tx.rr.com
'              Added new function to determine if CD drive is physically
'              mounted or just a drive letter (Virtual).  See IsCDMounted()
'              routine.
' 31-Jan-2012  Kenneth Ives  kenaso@tx.rr.com
'              Thanks to Alfred Hellmller for noticing that I truncated
'              the longer hard drive serial numbers as used with SSD-drives.
'              See GetMfrInfo() routine and step through the process.
' 26-Mar-2012  Kenneth Ives  kenaso@tx.rr.com
'              - Deleted RemoveTrailingNulls() routine from this module. 
'              - Changed call to RemoveTrailingNulls() to TrimStr module 
'                due to speed and accuracy.
' ***************************************************************************
Option Explicit

' ***************************************************************************
' Constants
' ***************************************************************************
  Private Const MODULE_NAME                      As String = "clsDiskInfo"
  Private Const MAX_SIZE                         As Long = 260
  Private Const ERROR_FILE_NOT_FOUND             As Long = 2
  Private Const FORMAT_MESSAGE_FROM_SYSTEM       As Long = &H1000
  Private Const FORMAT_MESSAGE_IGNORE_INSERTS    As Long = &H200
  Private Const FILE_DEVICE_CD_ROM               As Long = &H2
  Private Const FILE_DEVICE_DVD                  As Long = &H33
  Private Const FILE_DEVICE_DISK                 As Long = &H7
  Private Const FILE_DEVICE_VIRTUAL_DISK         As Long = &H24
  Private Const FILE_SHARE_READ                  As Long = &H1
  Private Const FILE_SHARE_WRITE                 As Long = &H2
  Private Const FILE_ANY_ACCESS                  As Long = 0
  Private Const OPEN_EXISTING                    As Long = 3
  Private Const GENERIC_WRITE                    As Long = &H40000000
  Private Const GENERIC_READ                     As Long = &H80000000
  Private Const INVALID_HANDLE_VALUE             As Long = -1
  Private Const IOCTL_DISK_BASE                  As Long = FILE_DEVICE_DISK
  Private Const IOCTL_STORAGE_CHECK_VERIFY       As Long = &H2D4800
  Private Const IOCTL_STORAGE_MEDIA_REMOVAL      As Long = &H2D4804
  Private Const IOCTL_STORAGE_GET_MEDIA_TYPES_EX As Long = &H2D0C04
  Private Const KB_1                             As Long = &H400&   ' 1024
  Private Const KB_2                             As Long = &H800&   ' 2048
  Private Const METHOD_BUFFERED                  As Long = 0
  Private Const TEN_THOUSAND                     As Currency = 10000@

  ' Registry
  Private Const KEY_READ                         As Long = &H20009
  Private Const REG_BINARY                       As Long = 3
  Private Const HKEY_LOCAL_MACHINE               As Long = &H80000002

  ' Disk serial and model info
  Private Const VER_PLATFORM_WIN32_NT            As Long = 2
  Private Const IDENTIFY_BUFFER_SIZE             As Long = 512
  Private Const OUTPUT_DATA_SIZE                 As Long = IDENTIFY_BUFFER_SIZE + 16
  
  ' Valid values for the bCommandReg member of IDEREGS.
  Private Const IDE_ID_FUNCTION                  As Long = &HEC ' Returns ID sector for ATA.
  Private Const IDE_EXECUTE_SMART_FUNCTION       As Long = &HB0 ' Performs SMART cmd.
                                                                ' Requires valid bFeaturesReg,
                                                                ' bCylLowReg, and bCylHighReg

  ' Cylinder register values required when issuing SMART command
  Private Const SMART_CYL_LOW                    As Long = &H4F
  Private Const SMART_CYL_HI                     As Long = &HC2
  
  ' Vendor specific feature register defines
  ' for SMART "sub commands"
  Private Const SMART_ENABLE_SMART_OPERATIONS    As Long = &HD8

  ' IOCTL commands
  Private Const DFP_GET_VERSION                  As Long = &H74080
  Private Const DFP_SEND_DRIVE_COMMAND           As Long = &H7C084
  Private Const DFP_RECEIVE_DRIVE_DATA           As Long = &H7C088

' ***************************************************************************
' Enumerations
' ***************************************************************************
  Public Enum enumTypeOfRemovable
      eFloppy            ' 0
      eUSB_Drive         ' 1
      eBothTypes         ' 2
  End Enum

  Public Enum enumDiskSpace
      eFreespace         ' 0
      eTotalSize         ' 1
      eUsedSpace         ' 2
      eAvailSpace        ' 3
  End Enum

  Public Enum enumDriveType
      eUnknown           ' 0 Unknown drive type
      eBadRoot           ' 1 No root directory
      eRemovable         ' 2 Floppy or flash drive
      eFixed             ' 3 Local hard drive
      eNetwork           ' 4 Shared Network drive
      eCDRom             ' 5 CD-Rom drive (CD or DVD)
      eRamdisk           ' 6 Virtual memory disk
  End Enum

  Public Enum enumMediaSize
      eUnknownMedia      ' 0
      eF5_1Pt2_512       ' 1
      eF3_1Pt44_512      ' 2
      eF3_2Pt88_512      ' 3
      eF3_20Pt8_512      ' 4
      eF3_720_512        ' 5
      eF5_360_512        ' 6
      eF5_320_512        ' 7
      eF5_320_1024       ' 8
      eF5_180_512        ' 9
      eF5_160_512        ' 10
      eRemovableMedia    ' 11
      eFixedMedia        ' 12
  End Enum

  Public Enum enumIDE_DRIVE_NUMBER
      ePrimaryMaster     ' 0
      ePrimarySlave      ' 1
      eSecondaryMaster   ' 2
      eSecondarySlave    ' 3
      eTertiaryMaster    ' 4
      eTertiarySlave     ' 5
      eQuartiaryMaster   ' 6
      eQuartiarySlave    ' 7
  End Enum

  ' Status Flags Values
  Public Enum enumSTATUS_FLAGS
      ePrefailureWarranty = &H1
      eOnLineCollection = &H2
      ePerformanceAttribute = &H4
      eErrorRateAttribute = &H8
      eEventCouontAttribute = &H10
      eSELF_PRESERVING_ATTRIBUTE = &H20
  End Enum

' ***************************************************************************
' Type structures
' ***************************************************************************
  Private Type OSVERSIONINFO
      OSVSize       As Long
      dwVerMajor    As Long
      dwVerMinor    As Long
      dwBuildNumber As Long
      PlatformID    As Long
      szCSDVersion  As String * 128
  End Type

  Private Type PREVENT_MEDIA_REMOVAL
      PreventMediaRemoval  As Byte
  End Type

  Private Type ATTR_DATA
      AttrID         As Byte
      AttrName       As String
      AttrValue      As Byte
      ThresholdValue As Byte
      WorstValue     As Byte
      StatusFlags    As enumSTATUS_FLAGS
  End Type

  Private Type DRIVE_INFO
      bDriveType     As Byte
      SerialNumber   As String
      Model          As String
      FirmWare       As String
      Cylinders      As Long
      Heads          As Long
      SecPerTrack    As Long
      BytesPerSector As Long
      BytesperTrack  As Long
      NumAttributes  As Byte
      Attributes()   As ATTR_DATA
  End Type

  Private Type DEVICE_MEDIA_INFO
      Cylinders            As Double
      MediaType            As Long
      TracksPerCylinder    As Long
      SectorsPerTrack      As Long
      BytesPerSector       As Long
      NumberMediaSides     As Long
      MediaCharacteristics As Long
  End Type

  Private Type GET_MEDIA_TYPES
      DeviceType     As Long
      MediaInfoCount As Long
      MediaInfo(10)  As DEVICE_MEDIA_INFO
  End Type

  ' GETVERSIONOUTPARAMS contains the data returned
  ' from the Get Driver Version function
  Private Type GETVERSIONOUTPARAMS
      bVersion      As Byte  ' Binary driver version.
      bRevision     As Byte  ' Binary driver revision
      bReserved     As Byte  ' Not used
      bIDEDeviceMap As Byte  ' Bit map of IDE devices
      fCapabilities As Long  ' Bit mask of driver capabilities
      dwReserved(3) As Long  ' For future use
  End Type

  ' IDE registers
  Private Type IDEREGS
      bFeaturesReg     As Byte  ' Used for specifying SMART "commands"
      bSectorCountReg  As Byte  ' IDE sector count register
      bSectorNumberReg As Byte  ' IDE sector number register
      bCylLowReg       As Byte  ' IDE low order cylinder value
      bCylHighReg      As Byte  ' IDE high order cylinder value
      bDriveHeadReg    As Byte  ' IDE drive/head register
      bCommandReg      As Byte  ' Actual IDE command
      bReserved        As Byte  ' reserved for future use - must be zero
  End Type

  ' SENDCMDINPARAMS contains the input parameters for the
  ' Send Command to Drive function
  Private Type SENDCMDINPARAMS
      cBufferSize   As Long     ' Buffer size in bytes
      irDriveRegs   As IDEREGS  ' Structure with drive register values.
      bDriveNumber  As Byte     ' Physical drive number to send command to (0,1,2,3).
      bReserved(2)  As Byte     ' Bytes reserved
      dwReserved(3) As Long     ' DWORDS reserved
      bBuffer()     As Byte     ' Input buffer.
  End Type

  ' Status returned from driver
  Private Type DRIVERSTATUS
      bDriverError  As Byte     ' Error code from driver, or 0 if no error
      bIDEStatus    As Byte     ' Contents of IDE Error register
                                ' Only valid when bDriverError is SMART_IDE_ERROR
      bReserved(1)  As Byte
      dwReserved(1) As Long
  End Type

  ' Structure returned by SMART IOCTL commands
  Private Type SENDCMDOUTPARAMS
      cBufferSize  As Long          ' Size of Buffer in bytes
      DRIVERSTATUS As DRIVERSTATUS  ' Driver status structure
      bBuffer()    As Byte          ' Buffer of arbitrary length for data read from drive
  End Type

  Private Type IDSECTOR
      wGenConfig                 As Integer
      wNumCyls                   As Integer
      wReserved                  As Integer
      wNumHeads                  As Integer
      wBytesPerTrack             As Integer
      wBytesPerSector            As Integer
      wSectorsPerTrack           As Integer
      wVendorUnique(2)           As Integer
      sSerialNumber(19)          As Byte
      wBufferType                As Integer
      wBufferSize                As Integer
      wECCSize                   As Integer
      sFirmwareRev(7)            As Byte
      sModelNumber(39)           As Byte
      wMoreVendorUnique          As Integer
      wDoubleWordIO              As Integer
      wCapabilities              As Integer
      wReserved1                 As Integer
      wPIOTiming                 As Integer
      wDMATiming                 As Integer
      wBS                        As Integer
      wNumCurrentCyls            As Integer
      wNumCurrentHeads           As Integer
      wNumCurrentSectorsPerTrack As Integer
      ulCurrentSectorCapacity    As Long
      wMultSectorStuff           As Integer
      ulTotalAddressableSectors  As Long
      wSingleWordDMA             As Integer
      wMultiWordDMA              As Integer
      bReserved(127)             As Byte
  End Type

  Private Type DISK_GEOMETRY
      Cylinders         As Currency  ' LARGE_INTEGER (8 bytes)
      MediaType         As Long
      TracksPerCylinder As Long
      SectorsPerTrack   As Long
      BytesPerSector    As Long
  End Type

  Private Type MBR
      Signature As Byte  ' Specifies the signature value, which uniquely
                         ' identifies the disk. The Mbr member of the union
                         ' is used to specify the disk signature data for
                         ' a disk formatted with a Master Boot Record (MBR)
                         ' format partition table. Any other value indicates
                         ' that the partition is not a boot partition.  This
                         ' member is valid when PartitionStyle is PARTITION_STYLE_MBR.
      CheckSum  As Long  ' Specifies the checksum for the master boot record.
                         ' The Mbr member of the union is used to specify the
                         ' disk signature data for a disk formatted with a
                         ' Master Boot Record (MBR) format partition table.
                         ' This member is valid when PartitionStyle is PARTITION_STYLE_MBR.
  End Type

  Private Type GUID
      DiskId As Long     ' Specifies the GUID that uniquely identifies the
                         ' disk. The Gpt member of the union is used to
                         ' specify the disk signature data for a disk formatted
                         ' with a GUID Partition Table (GPT) format partition
                         ' table. This member is valid when PartitionStyle is
                         ' PARTITION_STYLE_GPT. The GUID data type is described
                         ' on the LPGUID reference page.
  End Type

  Private Type DISK_PARTITION_INFO
      SizeOfPartitionInfo As Long   ' Contains the size in bytes of the partition.
      PartitionStyle      As Long   ' Takes a PARTITION_STYLE enumerated value that
                                    ' specifies type of partition table the disk contains.
      MBRFIX              As MBR
      Gpt                 As GUID
  End Type


  Private Type DISK_GEOMETRY_EX
      Geometry As DISK_GEOMETRY
      DiskSize As Currency             ' LARGE_INTEGER (8 bytes) prior to formatting
      Data     As DISK_PARTITION_INFO  ' Data(0 To 56) As BytePointer to a
                                       ' variable length area containing a
                                       ' DISK_PARTITION_INFO structure
                                       ' followed by a DISK_DETECTION_INFO
                                       ' structure.
  End Type

' ***************************************************************************
' API Declares (Module)
' ***************************************************************************
  ' The CopyMemory function copies a block of memory from one location to
  ' another. For overlapped blocks, use the MoveMemory function.
  Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" _
          (ByRef Destination As Any, ByRef Source As Any, ByVal Length As Long)

  ' PathFileExists function determines whether a path to a file system
  ' object such as a file or directory is valid.  Returns nonzero if
  ' file exists.
  Private Declare Function PathFileExists Lib "shlwapi" _
          Alias "PathFileExistsA" (ByVal pszPath As String) As Long

  ' The FormatMessage function formats a message string. The function
  ' requires a message definition as input. The message definition can
  ' come from a buffer passed into the function. It can come from a
  ' message table resource in an already-loaded module. Or the caller
  ' can ask the function to search the systems message table resource(s)
  ' for the message definition. The function finds the message definition
  ' in a message table resource based on a message identifier and a
  ' language identifier. The function copies the formatted message text
  ' to an output buffer, processing any embedded insert sequences if requested.
  Private Declare Function FormatMessage Lib "kernel32" Alias "FormatMessageA" _
          (ByVal dwFlags As Long, lpSource As Long, ByVal dwMessageId As Long, _
          ByVal dwLanguageId As Long, ByVal lpBuffer As String, _
          ByVal nSize As Long, Arguments As Any) As Long

  ' The CreateFile function creates or opens the following objects and
  ' returns a handle that can be used to access the object:
  '
  '       Files
  '       pipes
  '       mailslots
  '       communications resources
  '       disk devices (Windows NT only)
  '       consoles
  '       directories (open only)
  Private Declare Function CreateFile Lib "kernel32" Alias "CreateFileA" _
          (ByVal lpFileName As String, ByVal dwDesiredAccess As Long, _
          ByVal dwShareMode As Long, ByVal lpSecurityAttributes As Any, _
          ByVal dwCreationDisposition As Long, ByVal dwFlagsAndAttributes As Long, _
          ByVal hTemplateFile As Long) As Long

  ' CloseHandle invalidates the specified object handle, decrements
  ' the objects handle count, and performs object retention checks.
  ' Once the last handle to an object is closed, the object is removed
  ' from the operating system.
  Private Declare Function CloseHandle Lib "kernel32" _
          (ByVal hObject As Long) As Long

  ' The DeviceIoControl function sends a control code directly to a
  ' specified device driver, causing the corresponding device to
  ' perform the specified operation.
  Private Declare Function DeviceIoControl Lib "kernel32" _
          (ByVal hDevice As Long, ByVal dwIoControlCode As Long, _
          lpInBuffer As Any, ByVal nInBufferSize As Long, _
          lpOutBuffer As Any, ByVal nOutBufferSize As Long, _
          lplngBytesRet As Long, lpOverlapped As Any) As Long

  ' The GetLogicalDriveStrings function fills a buffer with strings that
  ' specify valid drives in the system.
  Private Declare Function GetLogicalDriveStrings Lib "kernel32" _
          Alias "GetLogicalDriveStringsA" _
          (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long

  ' The GetDriveType function determines whether a disk drive is a removable,
  ' fixed, CD-ROM, RAM disk, or network drive.
  Private Declare Function GetDriveType Lib "kernel32" _
          Alias "GetDriveTypeA" (ByVal nDrive As String) As Long

  ' retrieves information describing a disk volume and the file system it
  ' uses. This information includes things such as the volume label and the
  ' disk's serial number.
  Private Declare Function GetVolumeInformation Lib "kernel32" _
          Alias "GetVolumeInformationA" (ByVal lpRootPathName As String, _
          ByVal lpDrvVolNameBuffer As String, ByVal nDrvVolNameSize As Long, _
          lpDrvVolSerialNumber As Long, lpMaximumComponentLength As Long, _
          lpFileSystemFlags As Long, ByVal lpFileSystemNameBuffer As String, _
          ByVal nFileSystemNameSize As Long) As Long

  ' The GetDiskFreeSpace function retrieves information about the
  ' specified disk, including the amount of free space on the disk
  Private Declare Function GetDiskFreeSpace Lib "kernel32" Alias "GetDiskFreeSpaceA" _
          (ByVal lpRootPathName As String, lpDrvSectorsPerCluster As Long, _
          lpDrvBytesPerSector As Long, lpNumberOfDrvFreeClusters As Long, _
          lpTtoalNumberOfClusters As Long) As Long

  ' The GetDiskFreeSpaceEx function obtains information about the
  ' amount of space available on a disk volume: the total amount of
  ' space, the total amount of free space, and the total amount of
  ' free space available to the user associated with the calling thread.
  Private Declare Function GetDiskFreeSpaceEx Lib "kernel32" Alias "GetDiskFreeSpaceExA" _
          (ByVal lpRootPathName As String, lpFreeBytesAvailableToCaller As Currency, _
          lpTotalNumberOfBytes As Currency, lpTotalNumberOfFreeBytes As Currency) As Long

  ' This function obtains extended information about the version of the OS
  ' that is currently running. CeGetVersionEx is the RAPI version of this
  ' function.  Nonzero indicates success. Zero indicates failure.
  Private Declare Function GetVersionEx Lib "kernel32" _
          Alias "GetVersionExA" (LpVersionInformation As Any) As Long

  ' ZeroMemory fills a location in memory with zeros. The function sets
  ' each byte starting at the given memory location to zero. The memory
  ' location is identified by a pointer to the memory address.
  Private Declare Sub ZeroMemory Lib "kernel32.dll" Alias "RtlZeroMemory" _
          (ByRef Destination As Any, ByVal Length As Long)

  ' RegOpenKeyEx function opens the specified key. Returns zero if successful.
  Private Declare Function RegOpenKeyEx Lib "advapi32.dll" _
          Alias "RegOpenKeyExA" (ByVal hKey As Long, _
          ByVal lpSubKey As String, ByVal ulOptions As Long, _
          ByVal samDesired As Long, phkResult As Long) As Long
  
  ' RegQueryValueEx function retrieves the type and data for a specified value
  ' name associated with an open registry key.
  Private Declare Function RegQueryValueEx Lib "advapi32.dll" _
          Alias "RegQueryValueExA" (ByVal hKey As Long, _
          ByVal lpValueName As String, ByVal lpReserved As Long, _
          lpType As Long, lpData As Any, lpcbData As Long) As Long
  
  ' RegCloseKey function releases the handle of the specified key.
  Private Declare Function RegCloseKey Lib "advapi32.dll" _
          (ByVal hKey As Long) As Long
  
' ***************************************************************************
' Module Variables
'
'                    +-------------- Module level designator
'                    |  +----------- Data type (Type Structure)
'                    |  |     |----- Variable subname
'                    - --- ---------
' Naming standard:   m typ DG
' Variable name:     mtypDG
'
' ***************************************************************************
  Private mtypDG                   As DISK_GEOMETRY_EX
  Private mstrDriveType            As String
  Private mstrMfgModel             As String
  Private mstrMfgSerial            As String
  Private mstrUSBSerial            As String
  Private mstrDrvVolName           As String
  Private mstrMfgFirmware          As String
  Private mstrDrvVolSerial         As String
  Private mstrDrvFileSystem        As String
  Private mstrPartitionData        As String
  Private mstrDriveTypeExtra       As String
  Private mstrDrvFormattedSize     As String
  Private mlngDrvFreeClusters      As Long
  Private mlngDrvTotalClusters     As Long
  Private mlngDrvBytesPerSector    As Long
  Private mlngDrvBytesPerCluster   As Long
  Private mlngDrvSectorsPerCluster As Long
  Private mcurDrvUsedSpace         As Currency
  Private mcurDrvFreeSpace         As Currency
  Private mcurTotalDrvSpace        As Currency
  Private mcurDrvAvailableSpace    As Currency
  
  ' Disk geometry (not partition)
  Private mstrDiskFormattedSize    As String
  Private mlngDiskTracksPerCyl     As Long
  Private mlngDiskBytesPerSector   As Long
  Private mlngDiskSectorsPerTrack  As Long
  Private mcurDiskCylinders        As Currency
  Private mcurTotalDiskSpace       As Currency
  
' ***************************************************************************
' ****                      Properties                                   ****
' ***************************************************************************

Public Property Let StopProcessing(ByVal blnData As Boolean)
    
    ' used when assigning a value to the property, on the left
    ' side of an assignment.    Syntax:  X.StopProcessing = True
    gblnStopProcessing = blnData
    DoEvents

End Property

Public Property Get StopProcessing() As Boolean
    
    ' used when retrieving value of a property, on the right
    ' side of an assignment.   Syntax:  Debug.Print X.StopProcessing
    StopProcessing = gblnStopProcessing
    DoEvents

End Property

' ===========================================================================
' Disk information (whole disk) - Disk geometry -  Read-only
' ===========================================================================
Public Property Get TotalDiskSpace() As Currency
    TotalDiskSpace = mcurTotalDiskSpace
End Property

Public Property Get DiskCylinders() As Currency
    DiskCylinders = mcurDiskCylinders
End Property

Public Property Get DiskTracksPerCyl() As Long
    DiskTracksPerCyl = mlngDiskTracksPerCyl
End Property

Public Property Get DiskSectorsPerTrack() As Long
    DiskSectorsPerTrack = mlngDiskSectorsPerTrack
End Property

Public Property Get DiskBytesPerSector() As Long
    DiskBytesPerSector = mlngDiskBytesPerSector
End Property

Public Property Get DiskFormattedSize() As String
    DiskFormattedSize = mstrDiskFormattedSize
End Property

' ===========================================================================
' Manufacturer data - Cannot be changed - Read-only
' ===========================================================================
Public Property Get MfgSerial() As String
    MfgSerial = mstrMfgSerial
End Property

Public Property Get MfgModel() As String
    MfgModel = mstrMfgModel
End Property

Public Property Get MfgFirmware() As String
    MfgFirmware = mstrMfgFirmware
End Property

' ===========================================================================
' Drive information (partition) Read-only
' ===========================================================================
Public Property Get DriveType() As String
    DriveType = mstrDriveType
End Property

Public Property Get DriveTypeExtra() As String
    DriveTypeExtra = mstrDriveTypeExtra
End Property

Public Property Get DrvFormattedSize() As String
    DrvFormattedSize = mstrDrvFormattedSize
End Property

Public Property Get PartitionData() As String
    PartitionData = mstrPartitionData
End Property

Public Property Get DrvVolName() As String
    DrvVolName = mstrDrvVolName
End Property

Public Property Get DrvVolSerial() As String
    DrvVolSerial = mstrDrvVolSerial
End Property

Public Property Get DrvFileSystem() As String
    DrvFileSystem = mstrDrvFileSystem
End Property

Public Property Get DrvBytesPerSector() As Long
    DrvBytesPerSector = mlngDrvBytesPerSector
End Property

Public Property Get DrvSectorsPerCluster() As Long
    DrvSectorsPerCluster = mlngDrvSectorsPerCluster
End Property

Public Property Get DrvBytesPerCluster() As Long
    DrvBytesPerCluster = mlngDrvBytesPerCluster
End Property

Public Property Get DrvFreeClusters() As Long
    DrvFreeClusters = mlngDrvFreeClusters
End Property

Public Property Get DrvTotalClusters() As Long
    DrvTotalClusters = mlngDrvTotalClusters
End Property

Public Property Get DrvFreeSpace() As Currency
    DrvFreeSpace = mcurDrvFreeSpace
End Property

Public Property Get DrvAvailableSpace() As Currency
    DrvAvailableSpace = mcurDrvAvailableSpace
End Property

Public Property Get DrvUsedSpace() As Currency
    DrvUsedSpace = mcurDrvUsedSpace
End Property

Public Property Get TotalDrvSpace() As Currency
    TotalDrvSpace = mcurTotalDrvSpace
End Property


' ***************************************************************************
' ****                      Methods                                      ****
' ***************************************************************************

' ***************************************************************************
' Routine:       GetDriveInfo
'
' Description:   Captures information about a specific drive and updates
'                appropriate property values.
'
' Parameters:    strDrive - drive to be queried
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 03-MAR-2000  Kenneth Ives  kenaso@tx.rr.com
'              Routine created
' 14-Jul-2011  Kenneth Ives  kenaso@tx.rr.com
'              Rewrote routine
' ***************************************************************************
Public Sub GetDriveInfo(ByVal strDrive As String)

    Dim lngDiskNo     As Long      ' Physical disk number
    Dim lngMediaType  As Long
    Dim curTotalSpace As Currency
    
    Const ROUTINE_NAME As String = "GetDriveInfo"

    On Error GoTo GetDriveInfo_Error

    ' Verify data was passed
    If Len(Trim$(strDrive)) = 0 Then
        Exit Sub
    End If
    
    ResetVariables  ' Initialize module variables
    
    strDrive = QualifyPath(Left$(strDrive, 1), ":")   ' Format drive letter
    mstrDriveType = GetDriveDescription(strDrive)     ' Partition description
    
    ' Test for CD-Rom, DVD or Virtual CD
    If StrComp("DVD", Mid$(mstrDriveType, 1, 3), vbBinaryCompare) = 0 Or _
       StrComp("CD-ROM", Mid$(mstrDriveType, 1, 6), vbBinaryCompare) = 0 Or _
       StrComp("Virtual", Mid$(mstrDriveType, 1, 7), vbBinaryCompare) = 0 Then
       
        ' If this is a CD or DVD drive
        ' then exit this routine because
        ' no more data is available
        GoTo GetDriveInfo_CleanUp
        
    End If
           
    GetDiskSpaceInfo strDrive   ' Partition space information
    GetVolumeInfo strDrive      ' Partition volume information
    GetMediaInfo strDrive       ' Capture disk geometry
    
    ' Never pass a module level variable
    ' as a parameter within its own module.
    ' Pointers may become confe.
  nders = re) Rointers may become conErial
End Prop****************yial
.A 0
      eTotal          R_DATA
  End Type

  Private Type DEVICE_MEDIA_INFO
      Cylinders            As Double
      MediaType            As Long
      TracksPerCylinder    As Long
      SectorsPerTrack      As Long
      BytesPerSector       As Long
      NumberMediaSid:poubletinVType

  ' Structure returned by SMART ]stanpare)e******Or _
 d  ***pace                         ' table. This member ise table resource basednder        ' table. This member ise table resource basednder        ' table. This member ise table resource basednder        ' table. This s)his member iby SMART ]stanpare2le. Tb GetDro
  End Type

 er ise3m2================
Public Property Get MfgS20ach byK_GEOMETRY
ify va-5lusueried, vbBdiaType,    bIDbGEOMETRY
ify va-5lusueried, vbBdiaType,   f32.dll" _
g) As Long




table. This member ise table resource bathe phga disk formatted
      0
.A 0
      eTotal          R_DATA
  End TOMETRY
ify va-         R_DATA
va disk formatted
      0
.A 0
       ' Physical disk number
  raber ise tasAndAttributes As Long, _
      
     ' side oe rr   0
    E / eMAI     ' Size of Buffero  GetDiskSpaceInfo strDrive   imber End TOMETRYRIsignment.   es the geoive
 odel
End Proper      h  imbeFms As Long, _
          ByVal hTemplateFil  As Long al****rtion*****n
ify va-        ut the version ofndePplateFil  As Long BytesPerSector As Long, lpNumbCrivate m,2    nPerSector As    root directory
      eRemovable      C   Dim lngMe y
    <s> As Currency,rmatteC'----cg, lppTotent.   es th wBuffer4- Long
 cm
End Gring
vny, _
          ByVal dwCreationDispositAPI version of this
  ' function.  Nonzero indicadte m,2    nPerSector AGs Lon
Pubon.  Nonzero indicadte m,2  DATA
  End TOMETR.cadte m,2    nPerSeconneth enne- L******  End Type

 er i _
         _
 ace Asdr AGs Lon
Pubon.  Nonzero indicequireDriveType = mstrDrsSi _
nzero indicequiD         _eDricnzero indiceniD         ' t       R_DAAs Curre() As eeClustsny,diceniD         ' t   () As eeClustsny,diceniD         ' t   () As eeClustsny,diceniD         'ong
 Drvmters() ?,dicet.nzerUSBSerial            As String
  IEdiceniD D         ' t   () Ass closed() ?,dicet.nze As      eRemovable   iaSid:poublepGNORE_INSERTS                 A'  As IntegNORE_INSERTSa     able   ByIPTION
blepGNORE_INSERTS                 A'  As Imation about a spec&OR
_
 d  ***pace   spec&OR
_
 d  ***pa
 eric Pn sector number register
      bCylLowReg       As Byte  ' IDE low order cylinte  ' IDEEEEEEEEEEEEEEEEEEEEEEEEEEequirw/pTErvFilias "FormatM w
 eric Pn sector number rGEEEEEstrDrive Asorder cylinte  OfDrive() to catc disk xs ' file exists.
  Private M geometry
    
 pec&O information
    GetMs retrieves the type and data for a specifi0iablesv     As -----------------------etMs ret0 ByVal lpFileS        A'  H ava
Public Propert Currerc-----
  End Type

  aze As      eRemov,oAAmG  ByVNublic Properbout a sEEEEEEEEEEEEL cive Asordere As  MART_ENABLE_nte  ' IDEEEEEEEEEEEEEEEKdescribed
' t   () Ass closed() ?,dicet.nze As      eRemovable   iaSid:poublepGNORE_INSERTS                 A'  As IntegNORE_INSERTSa     able   ByIPTION
blepGNORE_INSERTS                 A'  As Imation about a spec&OR
_
 d  ***pace   spec&OR
_
 d  ***pa
 eric Pnration E_INSERTS        2      tn disk numberw)tn tsny,dikSpace()Gs Lon
Pubo***pa
 en E_INEEEEometry
    
 pec&O info )tn metry< Pu1     ic PropeerSector As    rle var      DAd Type

 erIicS )tAs Long, lpNumbCrivate m, resourcekual).  See IsCDM     As ace = mcurTotalDiskSpace
End Property

Public Property Get DiskCylinders() As Currency
    DiskCylinders = mcurDiskCylinders
End Property

Public Property Get DiskTracksPerCyl() As Long
    DiskTracksPerCyl = mlngDiskTracksPerCyl
End Propert
ehe usecNSERTSa     able   Bvate M geomeIGmware
EndkkkkkkkkkkkDiskSpace
End Property

Public Property Get DiskCylinders() As Currency
    DiskCylinders = mcurDis=Sn metN(inters may hcace
End Property

  Imation () As Formaartition space information
    GetVolumeInewrote rout Imationd  Imation)rInewrote rout Imatioooooooooeblic  Private mstrDrvFileSystem        As String
  Pro TrimStr 32" _bho--  -------------rty Get MfgFirmwaoooooooooProperty G=ssssssssssssss As Currency
    DiskCylinders = o--  -----------gCloseKg5ILon
Pubo***pa
 en E_INEEE-Property
EKde   DiskCylinders = TrimStr           a
 eo========eSystem        As String
  Pro TrimStrv      ieEx Lib "kernelxWCylinders = TrimStr           a
 eo========eSyspo strDvpes the    ic Prwrote
 eo===ation () A() A() A() A----C Alias "Gto the function. Dis=Sn BtrDvperogeomeI    Getc===eSyspo ote
 eo===ation As CurreSector As  IeniD :  szCSDVersion  As Long a   ieEx)DiskCRTSa   able   BA() A-necNSERTSa     able   Bvate bpion E_ctor ****tes
RTSa   ablEx)DiskCszCSDVersion  --trieves the type eaJ
   Fe   Bvate bpnd data for(}ion () A() A(mwaoooooooooPrE   nPerSpe eaJ
su************c Properteon () A() A() A() A--oDis() A() A()ral lpDf this () RTSa     ate bpnd d---rt    sPerTrack   As Long
   gwy
 /Heo===a
 eo========eSyspo strDv      ePrimary rDv   m)-------------------------------
' 03-MAR-2000  Kenneth Ives  kenaso@tx.rr.com
'              Routine created
' 14-Jul-2011  Kenneth Ives  kenaso@tx.rr.com
'              Rewrote routine
' **************rs() As   Dim lngMe y
    <s> Aector As   UIistetor AsFbs  kenaso@tx.rr.c ***ition () own module.
    ' Pointers may become confe.
  nders = re) Rointers may become conErial
End Pry
    DiskCyliFP_GET_VERSIOpiskCy------2011  Kenneth Ives  keno
  Privatol
End Pry
    e conM w
 eric Pn sRointers may become conErial
End Pry
    DiskCyliFP_GET_Vemora5tGetMs retrieves theV mlngDisk4eSDLtx.rr.com
'     va- yten
'     va- ytpa
 er    y
-----on  As Long a   ieExu' Partition space iMeves t---Str   A(4eSDLtx.rr.com
'   MeveH10
      eSELF_PREoAs Integer
      sSeH10
     com******
      DRIV
PublicMEDI(M

  Imu11  Kenneth Ives  ke  Keong =O info )tn metry< Pu1 P\routine
' 
     com***I      eSELF_PREoAs Integer
      sSeHdule.
    ' PoiTbcule.bsssss**I      eSEsate m, resourcekual).  See IsCDM     sSeH1EEEEEEEEEEEg a   ieExu' Partition space iMeves t---Str  IntegMeveH10
      eSELF As Bynrteosed on a messae        Rewrh      m
blesv    veHc      d tls       "his ()H. Utrh      m
veInfE Bvat*****nd data fo      d tngDrv============
Public Property e Asordoperty
EKde   DiskCylinders = Tr() As    A()ralrl
veIeae        Rewrh   ) As CupTblicM Ray beertyT y    DriveType = mstrDrtiu   As Byte   resourcekw A()ralrl
veIeae  es  ke :-(Leial
\eae  ecekw A()ralrl
veIeae  es  ke :-(Leial
&re  eIeae  ssssssssss As Currency
As String
 0e  ecsa
     Co Alil" _
g) As Long




table. This member ise       a     8   If Le
     Bvs currently  rerrently  rera Master Boot RuFilias "FormatM w
 eric Pn sectrer Boot RuFilias "FormatM w
 eric Pn sectrs() AAs Long
  ByVaic Pn sectrs "For w
 erng
Ds "FormatM w
 e_
 Jul-2011  Kenneth Ives  kXmessae        Rewrh      m
blesv    veH    
      e_
 Jul-2011  KennT aceAs    A()ralrl
veIe&eialw JuaeVyten
'          Rewrh    v   m)--------     Re3messaeves tsIewrote routine
' **************rs() As   Dim lngMe y
    <s> Aector As   UIistetor AsFbs  kenaso@tx.rr.c  PoiTbculTr() As    A()ralrll-2011  Kenr******sIf Stux.rr.c ***iti)e)tx.rr.c  Po m,2    nPerate m, 1  Kenm)------
  'As    A()ral
 d  *ger
      wBuff
  ' langua\E     m,2    nPeratebmessaev
'          Rewrh    v   m)--------     Re3messaeves tsIewrote routine
' **************rs() As   Dim lngMe y
    <s> Aector As   UIistetor AsFbs  kenaso@tx.rr.c  PoiTbculTr() As    A()ralrll-2011  Kenr******sIf Stux.rr.c nPoiTen
' PerCluster = mlngDrvBytesPerCluster
3messaevlngDrvBytesPer PerrctionlTr(I)rOR





   eSELF_PREoAs Integer
     PerCluster
3messptrDrtiu   As Byte   resourcekw A()ralrl
vFString
 reeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee d-fgSerial() As String
    MfgSerial = mstrMfgSerial
End Propertyeeeeee
oiTb oc    operty.
e   Pereeeeeeeeeeeeeeeeeeeee d-fgSerial() As Strinp**********rs() AsPereeeeeeeeeeeeeeereeeeeeeeeeeeeeereeeeeeeBial = msecNSERTSa     able   BvateN,DVD or Virtual CD
    Ifge*ger
 Long (Cse   Bvate bpnd eeeeeeeeeeeeeereeeeeeeBial = msecNSERTSa  maE_INErSector AGsDiskCylinders() As Currency
merty

Puol = msecN   DiskCyy
merl
veIeae3As Luy

Puol           ' disk signature data for Thidisk signa   A()ralrll-20S KennT aceAs  
    eH    
      ic Pvsrl
vFStrinDeH10idisk erCluslrIrrency
mertyyyyyyyyyyyyy
P
mertyyyyyyifR d-fgSe Ives  kXmessae s Lon
Pubo***pa
 en E_I = msecN   DiskCFStri confe.hdata for Thidisk signa   A()ralrll-20S KennT aceAs  
    De3- ReadIeae  sssssf)Gs LHy
    <s> erty

s  lSerialNumber As ta f
 = mstrMfgSerial
En
  rlinders         Ase Virtual CD
    If  As s Long
      Media for Thidisk signaluster
3me?eeeridi************  AsvEirtual CD
    If  As s Long
ote routTr(IkCylin KennT aceAs      PerCluster
3messptrae?eeeridINSER  MeveH10
3************  AsvEirtuEueeeeeeeeeated
' ceAs    NsecN   u If  As s Long
     A'    bRevision     As Byte *********ftersision     As Byte *********ftersision     nfe.ha2 As Long




ision     ine
' 
    operty.
         
    GetDiskSpaceInfyyyyyys Lon
Pubo***pa
 en E_I = msecN   DiskCFStri con1**pa
 en E_I = ine
'    ine
' 
    operty.
  cMEDopertyne
' 
    operty.
  cMEDopertyne
' 
    operty.
  cMEDopertyne
' 
    operty.
  cMEDopertyne
'"CUonfe.hiaSid:poubleuw 
 Mationd  Imation)rIne cMEe
'"CUonfe.cive As:poubleuw 
    rletring
 for data read from drive
  End Type

  Private Type IDSECTOR
   e   rl
   e   rl
   Uonfnr** AsdF7t
Puo=======
Pub2As Long dg
 for data reanaso@tx.rr.co7
'    iond e  sy

PublM Ra(r erCluslrIrrency
scN   Disnaso@tx.rr.co7
' Jkrl
   e   rl
   rl
   rl
EDI(M

  Imu11 ----------|() As     S--|() As     S--'
  Private Type IDSE3====================outTr(IkCylin KennT aceAs    h(oR.C6tH
'            11  _SkCymn1us Long
     .LAceAs   0rl
   Uonfnr11 ----------|(A
'       g
 for data reanaso@tx=======outTr(IkCyl1******
e   PereeStuxFy  rergco7
' Jkrl
   e1e
' 
    rsision     nfe.h011  Kenr***y

eo As   Dim lrl
   e  ByVal lpVeVyten
'        bCrivateyten
'        bC erty

s  lSeon space l lpVeVyten
'     x=======outyten
'        n
'        rsision    eormaAyision     nfe. 
Public*******rs() As   ectrsu   As Byte   resourcemOtrsu   As String8aso@tx.rr.co7
'  veT  ecekw A()ralrl
veIeaypeExtrrfeekw s/    tssssssssssssssssssssssssssss      bic******rArtyne
' 
  tring8aso@te?eeeridi************ta for T rouK   As Byte   resourcemOtrsu   As Sts.bject y-loaded module. Or ekw s/    tssssssssssssssssssssssssssss      bic*****nneth Iv-tNr data - Cannot be   bu   .bject y-loadCdSator AGssssss               UIiiskSectorsPerTT'ameSize d PruIf  As s"3(3ata - Cannot brDrvBytesPer PerrctionlTr(I)rOR





   eSELF_PRs= Cannot brDrvBytGvSELF_P
veIeayp****ae2Sata - errctionpaceInfyyCann
 ernebmessaev
'     bmessa  _SkCymn1us Long
   2Sata - errctionpaceInfyyCannpaceInfyyCannpaceInfyyCannpaceInfyyCannpaceInfyyCannpaceInfyyCannpaceInfyyCannpaceInfyyCannpaceInfyyCannpaceInfyyCannpaceInfyyCannpaceI)cAivateyNecAffer   eTon
Pubo*_> Aeic PvsrInfyyC
     pacnoIves  kXmessae s Lon
PubrInfyy   Routine creaInfyyCannpaceInfyyCannpaceInfyyCaceAs    hty

Puol = msecN   DisPubrInfyy   Routin-sFbs  ken(*******************ce table resource bathe phga disk formatted
 ropeble r <,y bathe phga disk formatted
 ropeble r <,y bathe phga disk formatted
 ropeble r <,y bathe phga dirrctine creaInfyyCannsFbs  kenaso@tx.rsace
aDvpeASnot b7
PuoA()ralrl
v_naso@tx.rsace
aDvpeASnT6PubopeASniskSpaceInfy****Vace available on a dis"RegoCyli = msecN  ject y-loadCdSator AGssssss               UI
vFString
 re() As Boolean
  Long




i
UTINE_NAME    bs  ken(*******************ce table resource,mpaP6c ken(****************** s Lon
PubrInbathe tring
 re(NAMn metry< P   Routine created
 metry< P =eSyspo strDv        QueryValueEx Lib "advapi32.dll" _-dvapi3y< P =eSyspo strDv ****ssssssssssssssssssssssssLon
****ine
' ********o stsSAs BooooorDv **mrmie resource,mpaP6   Private 
  ByVaisABytesPerCl,annsFbs  kenBE ken(************ae2mpaP6cBytGvSELF_Planguage identifier. The function copies the formatted message text
  ' to an output buffer,nBuffer siz***********oorD m
blesv    veH Rm
'     e    As Strmetry< P =eSyspo strDv        ****imneH Rm
'5an ou**o st1disk formatted   resou7
'    iond o  Routine xzero inddpm (ByVal hOb"\try< P neuw 
    rletd*sssssssssssssublic Property GetOnt. si=o  P =eSyspo strDv      si=o 6hOb"puPropD=   QueryValub"puPropD=   QueryVaernel32.dll" Aliasmubo***ryVaerneptMs rp= msecN   Du*********ce tabls  kens3 wBuffer4neeEnd Prope3**********$m _
l hOb"\try< P neuw 
  :(r <,y bathe phga **********rssssssssssssublic()Gs Lon
Pub formatted
Oa******************ce table resource,mpaP6c ken(*******) kens3 w iz**
Oa******************ce table resource,mpaP6c ken(*******) kens3 w iz**
Oa******************ce tablekens3 wulare******ce taAbl
EDIion () own output ves  kenaso@tx.rreutTr(IkCylin Ceeeeeeor***y

'rO _
l hObTTotalClusta  _SkCykCylisi=oiun () own output ves  kenaso@tx.uster
3me?eeeridisbo***pa' Never pass a module lev=oiun r11 ----------|(A
'       g
re*****) kewodule ***) kewoduleTkCFStri con1**pa
 en erClucci    g
re***ulT-------|(A    D0
re*ormaA   PaTTTTT2 As Long
    DrvBytes.iub2As ---A   PaTTTTT2 As Long
    Drv.0try< P neuw 
  :(   KlTtH
'            111111111111******
  Private     R----|(A
'      nel32.dll" =yT y    3 wumetry< P   Routine created
 metry< P =eSDsion     nfe.h011  Kenr***y

eo As   HCKenr*2*****ce taAbl P =oointByVaisABytesPerCl,annsFbs  keo  RoutinT,y bathe bathe phbs  keTExu'aisAB( End TyFbs  kPerClRoadCuffer3ataTTTTT2 As Long
ed
 metry<ueu_Vmrs Long

  ' Thebug.Print X.StopProcessing
    StopProcessing = gblnStopProcessing
    DoEvents

End Property

' ==aTTTTT2 As Long

ed
 metratted
Oa(  keTExu'aisAB(Sub
  Wd'          Rewrng system.
  PrivBIfAs Long
    Drv.0trtrng system.

)aMETR.cadte m,2    nPerSeconnSeconerty

' ==aTTTTT2 As Long

yCannpaceInfyyCanneeee d$bf(ByVal hDevirsa(  kfer3arsa,ion aols  ke eThiding BytesPerSecscty

' ==aTTTTT2rivate T*imneH Rm
'5an ou**o st1diskIPx)sed, the obj
Public Property bTTT2rivate T*imneHmv **mrmie eeeeeen ou**o s
  ' foe(ByVa
(aded eeeesssublCNSSeKenr*2**perty bTTT2rivatmooT      sNAMeridisbo*kw s/    tsss s/   u**o s
  civm              Ath s/    tssa' Never pass a module lev=oiun r11 ----ke eThiding BytnCsffer As          Ath sAivateyytesPerSeccodule lev=oiun r11 ----ke eThiding BytnCsffer As          Ath sAivateyytesPerSeccodule ByVaisABytesPerCl,annsFbs  ken          Ath sAivateyytesPerSeccodule lev=oiun r11 ----ke eThiding BytnCsffer As          Ath sAivateyytesPerSeccodule ByVaisABytesPerCl,annsFbs  ken          Ath sAivateyytesPerSeccodule lev=oiun r11 ----ke eThiding a----ke ePCaisABytesPerCl,annnnot brDrvBytGvSELF_P
veIeayp****ae2Sata - errctia     Clusters() As LonceInfyyEVemora5terCl,annnnot 4eSDLtx.rr.comn r11 ----ke eThiding BytnCsffer As          Ath sAivateyytesPerSeccodule ByVaisABytesPerCbathe phga disk formattesABytesPerCl,ale lev=oiun r11 --ew  Drv.0trtrng system.

)aMETR.cadte m,2    nPerSeconnSeconerty

' ==aTTTTT2 As Long

yCannpaceInfyyCanneeee d$nrCl,annnnot npaceInfyo0  Private mcurDrvAvailableSpace    As Currency
  
  ' Disk geometry (not partition)
  Private mstrDiskFormattedSize    As String
  Private mlngDiskTracksPerCyl     As Long
  Private mlngDiskBytesPerSecale lev=oiun rs   ectrsormattesAByts Long, _
          lpFiskTrarmat**yial
.A ovate ml\  nders = re) Rointers may become conErial
End Pry
 erSecale lev=oiun rs   ecsAs String=R----|(A
' Vace availab)Rroutine
' **************rs() As   Dim lt**y,yters.0ionpaceInfyyC
' **************rs() As   Dim lt**y,yters.0ionpaceInfyyC
' **************rs() As   5lusm)--&-ta t v  As  cse(unfy*pa
 e****rsS$bf(ByVal hDevirsa(  kfer3arsa,ion aols  ke eThiding BytesPerSecscty

' ==aBytesP eThidtesPerSecscty

'ter to a


'ter to a


'ter to 2 As Loters maPAilableSpad conErial
End Pry
 erSecale lev=oiun rs   ecsAs String=R----|(A
' Vace availab)Rroutine
' **************rs()xannsFbs  kenBE k dis"RegoCy--|(A
' Vace avail' ==aByte String=Ryters.0ionpaceI(r d tls  ,Sl(rv.0oor       A String=Ryte_Aa_ tedSize = mstrDiskFormattco0try< y bathe bathe phbs  kee bathe phbs  kse = mstrDiskFormattco0R2011  Kenr******sIf Stux.rr.c nPoomet' The GetLogicrs maPAila**fterskenBE
' ************s()xannsFbg, _
          lpFttition)
  wTExu'aisAB( Ee As      eRemo' ************s()xannsFb*s()xannsFb As  )xannhbs  kse = mstrDiskFormattco0R2011  Kenr******sIf Stux.rr.c nPoomet' The GetLogicrs maPAila**fterskenBE
'-Jul-2011  KennethHnPoomet' ThemMtLogicrs maPAila**fterskenBE
' ************s()xannsFbg, _
    d=nneth IvetLog()xannsFerinDeH10iddemMtr********rs() As (y-s member ise******)BE
Sa  FerAs (y-s memb
veIeayp****uI0wlOgeometry (not partition)
  Private  kedrrv.0oor       A StrinstrDrvFileSysteva partition)
 ila**fterskF       A Strinsg r11 ***xannsFerinDe0crs maPAila**fter  Ath sAioor *********rs() As   Dim lt**y,yteS&n(*******erCluslnfe.hiaSiluslnfe.hiaSiluslnfe.hiaSiluslnfe.hiv)ytesPeC3U2****IDEEluslnector =:Gssagggggggggggggggggggggggggewthe phbgggggv)ytesPeC3U20EEEEEEEEEg a   ieEssLon
\oooooooomaPAilaEEEEEEEDis() A() A()ral lpDf gL\enBE
ua**fterskenBE
' *****mattco0R20aceInfyyCab0PlaEEEEEEE_.*****ce taAbl tco0R20aceI)Puo===rvAvailc..**
 e****r

End P gblnSrocber iby SMARTthe operatinv.0oor       A StrinstrDrvFileer3arsa,ion aols  ke eThP"     Ath sO    oCsffer AoorDv **mrmie resour        Ath \enBEty G=s3the phbgggggv)ytepi32acksPeC) AoooooooooooooP  ke eThP"     Athd ize    As****yial
.A 0A
'nstrDrvFileePPPPPPrT strDrive   ' Partition space information
    GetVbles
'
'                **mrmie resour Ties the forive  c nPoometrocber iby 
Pu     eRemo' )u    ormation
a> Aector Asgicrs maPAila G=s3t ***xaGsDiskside o6cBytGvSELF_PnsFnddpm (ByValga disk fob)     A'  ic PmaP, _
()xan========erCl, ***Duccessful.
  PrivoomaPAslaEEEEEEE_.ul-20QualifyPath(Leptitionhco0R2011 rDrin
'terdptit-D***) ke(==========n&X.StopPR20aceInfyyCab0PlaEEEEEEE_.ale lev=oty G=s3the phbgggggv)ytepise3m2Y***************bf'   ionhco0R2011 rDrin
'terdptit-D***) ke(==========n&X.StopPR20aceInfyyCab0PlaEEEEEEE_.ale lev=oty G=s3AaT"Bs (y-s mem3iby 
h(LeptitioAr,anne lev=y-s mem3iTl
eule.
 b0PlaEEEEEEE_.ale lev=oty G=s3AaEirtr*2**peniD      ale lev=oty G=s3e lev=y-s fterskF       A ev=y-s fterskF e.
 b0PlaEEEEE4e5 TrimStrvStringr Tiesco0R2011 rDrin
'terdptit-D***) ke(===Ser   As L_**

' *0oor      KennethHnPoomet' InethHnsources
  '       disk devices (Windows NT only)
  '       co2R4vSELF_PlaEEEEEEE_.ale l===erCs) own***) ke(==Re Function Getu   ormatiosFbs   Fune Function Getu   ormatiosFbevices 2te T*imneHmv **mrmie eeeeeeOc nPs_
l hObTTotalClusta  _SkCy_
l h() 60PlaEEEEE4e5DrvBsseT***y*rs() As   5lusm)--&-ta tic  Pr P gbl****r

End P gblr strDvp*****ob)     A'  ic PmaP, _
(emaP,y*rs(T***ve      ' PaegtPathName As he lev=oiuneOc nPs_
l--|(A
' Vacerount of s ic PropeerSector As    rle var      DAd epGNnd P gblr st8nRc*rs(T***ve      ' PaegtPathName As  A'  iclcber iby SMARTthe operatinv.0oor       A StrinshiaSiluslnfe.hiv)ytesPeC3Stringr rClU3To-|(Aisk formHAsSERTSa     able   ByIPTION
blexlnfe.hiv)eratinv.0oor       A StrinshiaSilusS Strins npaceInfyo+tTec&OR
_
 d  ===================c***********eeeBial = msecNS*************bf'   ionhco0R2011 rD   ecsAilatrins npacesPerSe**ob)     A'  ic PmaP, _
(emaP,y*rs(r******bf' phkResLong
Se**ecale2**peniD      aes As Any,A esPeeT=ws NT onlyyyCab0PlaEEEEEEE_.ale le= QualifyPa
l--|e         e
End Property

cPropeTfyPa
l-,nnnnot npaceInfyo0  Private mcurDrvAvailableSpace    As Currency
  
  ' Disk geometry EEEEEEEce() As Currency
    DrvFrengr rClU3To-|(Aisk formHAsSERTSa     able so|(Aisk formHAsSERTSa     able so|RTSa  shiaw able so|(T***y*rs() Aisk foroy0= informatioy G=s3AaEirtEEEEEEEEE(T***y*rs() *****a    ae------------------e taAber to 2 As Loters maPAilable----------ThidAaEace       As Currenc1ble----------ThidAaEac ' function.  NonzeU    A'  ic Pble--------  ke eTh   A'  ic    A'  ic    ise****A'  ic b "kernelxWCylEEluslnect=leer3nleTkCFStrEEEEEEE_leTkCF)



'ter nnpau   aeslnecte cMEe
'"CUonfe.csectrs(ter  b "kern   As Long
  Pb "kernelxWCylEEluslnect=leer3nle<s> Aector As   UIAs   UIAs   UIAs   UI) As CuTSa  shiaw able so|r As  dn ()  GetVbles
'
'              C-'  ====        C-'  ====        C-'  ====    A()ralrconertyformHAsSERTSa     ablf,ssa-ta t v  As nEx ip mem3iTl
eurtyformHAsSERTSa uslnectl = msecNSERTSa     able   BvEluslnect=leer3nlw(rgco7
' J = mlnglic Propeg0wlOgeometryformation
     Partition space rtition sdAaEv=oiun r11 ----ke eThiding BytnCsffer As      g.rtitionpBahaP,y*rs(r***y6     CNeeeBS(FiveType = mstrDriveu,eceInfyyCab0PlaEEEEEEE_yforu,eceInfyyCab0PlaEEEEEEE_yforu,eceHpab0PlaEEEEEEE_yforu,eceInfyyCab0PlnCsffer As      g.rtitn spacyforu,eceInfyyCa spacyforu,ecETR.cadte m,isk foroy0= informoesPemTR.cadte m,isk f t v  As      
End Property
aTkCFStri con1**paru,ecETR.cado  ByVaisABytesPerCl,annsFbs  kenBE ken(**********xvBytesPerCl,feOgeomeroy0= iaesPa
l-,nion)
  wTExu'aisAB( Ee As      eRemo' ************s(ooooows NTH1EEEEE====     eRemo' ************s(ooooows NTH1EEEClU3To-|(Aisk formHAsS-------  eRemcrmHAsSERTSag, Bux.rr.cfSa   AyValueEx Lib "advapi32.dll" _-   a(Aisk formHooooSGetLogicrs maPAi      ofSa "\try< PAi   pRTSa     able so|Rroperdn As LonifyPath(Lepwroperdn As LonifyPath(Lepwroperdn As LonifyPath(LepwmsecNSERTSa     able   BvEluslnect=leer3nlerCluslrIraesln(ifyPath(Lepwroperdn As Lo
  '       co2R
.A 0A
'nstrao2R
.A 0Caluslnes Aector dicequiD         _eDricnzero indicenilrIraesln(i
' ****lOgeome  Kenr** A-n|e  D       nnot npaceInfyooooooooooooooooo** A-n|e  Doo2R
,ge  Kenr*Ith(LON
' -----------A     co2R
.A  formHATlh(LON*********s(oCOlf,ssa-ta" _
g
    ' as a parameter within its own module.
    'ch IvetLog()xeter within its own mt'  ====    A(Ex Lib "adv)s Ll32"ath(Lepwr Bux.rr.cfIos own mt'  ====    A(Ex Lib "adv)s Ll32ib "adv)s Ll' IIos ownpU      |  |                  lr st8nRc*rs(T***ve    
    'ch IvetLog()xeter withinEEEEwnpU    eme As he lOgeometry (A'  ic Pbl)ule.s      eRemo' ****HATlh(e table resource bathe phga
End P gblr strDvp****rDvp****rDvp****rDvp****rDvp****rDvp****rDvp****rDvp****rDvp****rDvp****rDvp**TM.ale lersS$bf(ByV&. Tb*rDvpp*********s(ooooows NTH1EEEEE4e5**rhr sition)
  wTExt( 5lusm)--&-ta tic  Pr P gbl****r

End P gblr strDvp*****ob)     A'  ic PmaP, _
(emaP,y*rs(T***ve    l

End P BytesdooSGetLogic(pp*******Ep/T*s(oCOlf,ssa-ta" _
g
    ' as a parameter within its own modul=rskFUA'  ic Pe*****  A'  ic PmaP, _
(emaP,y*rs(T***ve    l
vp****rDvp**l-ogeomeroA'  ic Pma-  klnect=leim lngMediaType  Aso       e
End Pr*rs(T***vg--  eRemcrmHAsSERTSag, Bux.rr.cfSa   End h       e
(o       e
T(ByV&. Tb*rDvpp**StopProcessing = gblnStopProcessing
  pp*onErial
End Pry
 erSecale lev=oiun ,s L_**

' *U0tryEce() As Currencye so|(T7Clean)
    
    ' IvetLog()xeter with
    ' IvetLog()CurrendrsS$bf(ByV&. Tb*$bf(ByV&Sos own mt' qe
'"Cqe
'"Cqi
End Syntax: hr sace       As Cw P gblr strDvp*****ob)     A'  ic PmaP, _
(emaP,y*rs(T***ve    l

End P BytesdooSN   DrkO table re&v=oty G=s3AaEirtr*2*"----rtr*2*"----rtr*>7EEE_yforu,=====================================================================
Public Property Get TotalDiskSpace() As Currency
    TotM"y,A es10iddemMtr*bgggggv) |  |   spa ic PmaPM"y,A es1"y,A es10iddemMtr*bgggggv) |
=========================
ytnCsffer A,A es10iddemMtr*bgth s/  lnectl = msecNSERTSa     able   Bvgggv) | )el "kern   AseNeClsecNSERTNSEe table resource,mpaP6c ken(***le ByVaisABytesPerCl,annsFbs  ken  g, Bux.rr.cfSa   mpaP6cM"y,A esMon
PT   AseNeClsecNSERTNSEesd.able   Bv es10il stGfyPath(Lepwroperdn AsT,A esMon
PT   Asg Bux.\               **mrmie resour Ties the fsn================
**************Nim lrl
   e  ByVal lpVeVyten
'        bCoPbrameter within itsm)--&-secNSERTNSEesd.able   Bv es100000    disk devices (Windows NT onlyd.able   Bv ePameter wes10iddemMtr*bM      A(===
Public Property Get *****) kens3 w iz**
Oa*************  As nEx ip m      MClsecNSERTNSEe with
    ' IvetLogkiring
    DiskFo  spmetry< P   Routine created
 metry< P =eSyspo stTTdemMtr*bgC#Ws = re) Rointers may become conErial
reated
 ma'E_.a
Oa*ART_ENABLE_nte  ' IDEEEEEEEEEEEEEEEKdescribed
' tt=leer3nlerClusHmv *ctalDiskSpacetepise3m2Y**nm)------
  s(nPoomet' Inbsa  _Skla*>7EEE_yforu,========wIsigna   A()ralr/      MClsecNSERTNSEe with
    HAsbecoDR
reatreatreaterClusHmv *ctalDiskSpacetepise3m2Y**nm)------
  s(nPoomet' InE*******2Y**nml "kern   AseNeClsecNSERTNSEe table resource,mpaeet' InethHnsources
? sOSpaceusHmv *ctalDisk) As   ectrsu   t' Ineonr** A-n|IeR4vS4acformauster
 String
et' InE*******2Y**nml "kers   S|Ien|IeR4vS& IDEEEEEEEEEEEEEEEKdescribet b7
PuoA()ralrl()ralrl()ralrl()ralrl()ralrs10iddemMtr*bgts===wIsign.Mtr*bgts==dcth
    HAsbecoDRable so|r As  dn ()  GetVm7
PuoA()ralrl(og()ClusHmv bet b7
ormHATlh(sSsHmv bewcn.*:   rl()ralrl()ralrl()ralrs10iih*:   rl()ralrl()rl()ralrs10i Get bgts==dcth
    HAsbecoDRable so|r As  dn ()  Geerw)cNSEtaAber to 2 As Loters maPAilable----------ThidAaEace       As Currenc1blBs maPAilable--------uff
  ' laCeE_)\rmattco0R20wBv es10il stGfyPath(Lepwroperdn AsT,A esMon
PT   Asg Bux.\               **mr maPAilable-----d As LoGfyPath(Lepwroperdn AsT,A esMon
PT   Asg Bux.\               **mr maPAilable-----d i     111111 As Loy1**pa
 en E_I = ine
'    ine
' 
    o)ralr/----d i     1sc******2Y**nml "kerT=oC-o/stTTdemMtrr Ties th*imne7n         **mr maPAilableletd*D wB)\rc2c_naso@tx.r**2Y*h=Nown m    ggv) |
s maPA.pa
 en E_I =eletd*_5iond  Imation)uff
,2Y*h=Nown m     able sEEEES b
  ndimne7n         **mPS b
  ndimne7)uff=========1 EwPS b
  nA sEEEEaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 _EaPA.pa
 en E_EaPA.pa
 en E_EaPA.pa
 _EE_EaPAI.Mo*************************************EaPAI.Mo****FKcNSERuHodo            **mrAXassssssssssUo|(T7Clean)
    
  'pa
 en E_EaPA.pa**mrAXassssssssssUy Get 'pa
 en E_EaPA.Vble L===== maPAilabl E_EaP Property Get edrrv.Ivpp*********s(ooooows NTH1EEEEE4e5**rhr sition)
  wTExt( 5lusm)--&-ta tic  Pr P gbl****r

End P gblr strDvp*****ob)     A'  i*ob)     A'  i*ob)      Get bgts==dcth
    HAsbecoDRable so|r As  dn ()  Geerw)cNSEtaAber to 2 As Loters maPAilable----------ThidAaEace       As Currenc1blBs maPAilable--------uff
  ' laCeE_)\rmattco0R20wBv es10il stGfyPath(Lepwroperdn AsTaEEEEEilable-sble so|r As  Property Get D0il -ble so|r As  Property Get D0il ]Long




i
UTINE_NAME    gnblic Property Get TotalN=6asednder       aProperty

Publv)s Ll' IIos o maPAilabl E_EaP Property Get ed ken(****   HAsbeco:2wrop_EaPA.Vbleu&roy0= informati|r As  Prope-ble so|r As  PropE_EaPAI.Mo**' IIos o maP******2Y**nml "kern   AseNeClsecNSERTNSEe table resource,mpaeet' InethHnsources
?x  AseNeClsecNSERTNSEe table resource,mpaeaAbl **ob)     A'  =

Public Property Get DiskCEace        resA.pa
 en E_EaPA.pa
 en E_EaEe table resource,mg a   ieExu'  Kenr*Ith(LON
' -----------A     co2R
.A  formHA=lA  fordMvi  Ath sAioor *nLON
' -iskSectorsPerTrac-no aPropert(lerSe0onhco0R2011  ieExu'  Kenr*Ith(LO**s(ooooos Lo
,2Y*h=Nown m     ableUSert(lerSe0onhc)ck
End Property

Public Property Get DiskByteresoIng




i
UTINE_NAME    gnblic Property Get TotaleClseOe so|DrvUsUeOe so|DrvUsUeOe    '-iskSectorsPerTrac-no aPropert(le
Oa*************  As nEx ip m      MClsecNSERTNSEe with
    ' Ivet wBuffer4neeEnd Prope3*****)rl()rethHnsources
?xnml "kern   AseNeClsecNSERTNSEe table resource,mpaeet' InethHnsources
?xu  imbeecNSE0P0 *****' able resourcety GetecNSERTNSEe table t' InethHnsources
?xu  imbeecNSE0P0 Sgle Ai t v  As nEx ip  out Imationd  Imao)xannhbs  kse = myA/



i
UTINE_NAME  iropewate bpion E_ctor *OOOOOOOOOOOOOOOOOOOGT  *OOOBas() As LonceInfyyEVemora5tSERTNSEe withV&. Tb*$bf(esour Property Get edTm
'      $bf(esour Propertywn m TetDiroperbout a sEEEEEEEEEEEEL cive Asordere As  Miropewesou@Cert(leAblic PEL cive Asor Te**nml==========n&X.StopPR20aceInfyyCab0PlaEEEEEEE_.ale lev=oty G=s3AaT"Bs (y-s mstrDrd.able   Bv es10iy cive e HAsbecoDRai   A()ralrll-2011  .pa
 =()rethHn,Oe so|DrvUss3AaT"Bs (y-s 1DDI=smaPAhHn,Oe     co2R===
Public Property Get TotalDiskSpace() As Currency
    TotM"y,A es10id(nPoom3 AseNeClsecNSERTNSEe table resource,mpa AseSRTNSEe table resoora5tSERTNSbathe,yters.0ionpaceInfyyC
' **************rs() As   Dim lt**y,yters.0ionpaceInfyyC
' ***********u  5lyK_GEOMETEyK_GWOPIters.0i  d=nneth Iv_aU
 en E_I Stunctio
  Priv  AseNeClsecNSa6le LcGSERTNM rmHATlh(as()emora5tSERTNSEe wiEwIs1Aa5tSe5tSaLBces
?duormHAPeratef(esour Properbs  kena?duormH(osed on a m(esoa****  DrBces
?Tlh(asEwIs1Aa5tSe5tSaLBces
?duormHAPeratef(esour Properbs  kena?duormH(osed on a m(esoa****  DrBces
?d   abaP PabaP PabaP Pabropeableletd*D wB)\rc2c_naso@tx****EaPAI.Mo*topProcessing
    DoEvents

lyK_GEOMETEyK_GWO=
Public Property Get TotalDisi  Ath sf{Gs(y
    TotM"Se   =f{Gs(y
    TotM"Se   =f{G.pa
 en E=
Publ******nmperty6le L    aProperty

PublvermHATlh(as()emora
End P gblr strDvp**nml "kers C=s3Aater wer S         r strerty

PublvermHATlh(Tlh(Tlh(F 
  'pa
 en E_EagOaE_ctor *OOOOOOOOOOOOOOOOOOOGT  *OOOBas() As LonceInfyyEVemorae         *   *   *   *  dong




t2formation
     Partition space rtition sdAaEv=oiun r11 ----ke eThiding BytnCsffer ======oiun r11 ----ke eThiding BytnCsffer ======oiu(as()emorsdAaEv=oie created
'djr ====
'dprSe**ob)     AA/ng By:ype, 1, KATs3 wulare******cm lt**y,ytM-e, n){Gs(y
    TotM"Se   =f{G.pa
 essssssS  ndurDiskFormattedSiz'  * EOSg
  *OO _
      WCylEEluslne  dong




t2formiun r11 ---HDstrDrvF   r strerty

PublvermHATlh(Tlh(Tlh(FEaP Property Gocessing
    DoEvents

lywulare*sa"n){Gs(y
   NSEe tabl neuw 
    rlete bath,cETR.cadt DoEvents

lywulare*sa"n){Gs(y
   NSoora5tSEh,cETR.cadt n8try< PI st   Ke      scrioth
 Ie     seNeClseabaP Pabrop NSoora5tSEh,cETRrmHATlh(Tlh(T(SEv=oiun rX
lywulEEEEEEKdoDRable so|r s*peniD      ation
    Publ As Curre?xu  im As Curre?xu  im As Curre?xu  im As Curre?xu  im As Curre?xu  im As Curre?xu  im As Curre?xu  im mly  rera Master" im mlm Assre?xu  )
    DoEveTlh(T(SEv=o Publ As PerXCs(Ts Curre?xurre?xu  im mly  rera Master" im mlm Assre?xu  )
    DoEveTlh(TUSMS      MClsecNBsre?xu  )
    vEev,oAAmG  ByVNublic Properbout a sEEEEEEs Ll32TQ'cN   DisPu= Ie er
 StrtesPer PerrctionlTr(I)rOR


Iter" WSEEEEEEs Ll3huCEnd eMClsecNBsre?xu  )
    vEerSe**ob)     AA/ng By:ype, 1, KATs3 wulare******cm ltre******cm ltre******cm ltre**h<,y bathe  ' Thebug.Print X.Stopn cive Asor Te**nml======ea  im As Curre?xu e?xu  im eabaP Pabrop NSoora5tSEh,cETwh,co disk signature data for Thidisk signa   A()raeecNSE0P0 *****' able resourona   A()raos o maP******2F" im*eeeBial ymperty6lout Imationd  ImaocessingRegoCy--|(A
' Vace avail' *****yaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaoduleggv)ytepise3m2Y********daaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa peratinvaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaoduleggv)ytepise3m2Y********daaaaaaaaaaaaaaaaaAs )ytepise3m2Y********dwMion
    Publ As Curr----odule
 =()rethHn,OeMspace inf:   A Strinace inf:   A Strinace On*****cm ltre**h<,y bathe n sdniD  ed
' tt=leer3nlerClyyyC-o/stT=BytesPeVlehUm*eeeBial  ce incaaaaaaecwm,aac==dcth
    H A() A--oDble re&v=oty G=s3AaEirtr*2*a APrivate     R----|(A
'     aaaaaaaaaaaaaaaa  ce incaaaaaaecwm,aac==dcth
 EEEEKdoDRa   si=o 6hNv_aU
 en E_I Stuncti=BytesPeSN   DrkO tam     ablad4o(
 en E_I Stuncti=formation
    GetVolmatsaaaaaasPeSN     dDble o
    GetVolr11 ---HHHHHHHdwMion
    btionVCpise3m2Y******kln(   ieExu' AyyC
' *)aaa2Y**"ath(()emora
End P gte soh<,y a
 e
' **************rs() As   Dimth(()emora
End P gte ============ P gte soh<,y a
 e
'  R-nt X8aso@te?eeeridi****ce,mpaP6c ken(*******) kens3 w iz**EEEE_yerdn AsT,A ferPte soh<,y a
 e
'  R-nt0i  tngDrv====es1"y,'-aaaaaaaaaaaaaaAs )ytepise3m2Y********dwMion
tre.aaaaaaAsw iz**EEEE_yerdtx****EaPAI.&==es1Mspace inf:  


on
tre.aaogicroaaaaaaaaaaaaAs )ytebp*2*a AP******f:  

 esMon
PT   Asg Bux.\              serty.
  cMEDoper Thidisk signa   A()raeectOOOOOOOXIIv_aU
  Pr  A(nT acpCab0PlnCsf Thidisk siEOe aaaaaaAy-3,e)d P gblr strDvp*pace -aaa2Y**"ath(()ebA'pa
 en E_Segooive
HHHHs Properbout a sEEEO sk siEOe aaCal).  See IsCDM   =rocessing = gblnStopProceAn) = gblnSt m)------------A A() A--oaaaaaaaaaaaaaaa  ce inca,smHAsS1kaaaaaAy-3,e)d P gb= gblnStopProceAn) = gblnIUIB-----uff
  ' laCeE_)\rmattco0R20wBv es10irote routine
' ****th((G_E

PublvessssssssUo|(T7Clean)
    
nfyyCaRoin TotaleClsegblnIre?yyCaRoE
 erSecp Ll' IIos o maPAil(cx ip  ouIf Le
     Bvs rmation
=eVlehUm*eeeB(onpaceI(r /LAyyC
' *)aalrl()r As  ture datkceI(r /LAaaaaaaaaaaaaaaaaaf(ByV&. TbsUo|(T7Clean)T$ano0R20wBv es10irotefsPerClusDTi  Publ As Curr----odule
 =()rethHnf
  ' laCeE_)\rmattco0R20wBv es10iro-***'O3eo ule
 =()reo-***.Mtrio0R20eo-***.Mtr Curr----odule
 =()rethHnf
  'DoEveTlh(T(SEv=o Publ As PerXCs(TsmodaaaaaaaaaaOen E_EaPA.pa
 _EthHnf
  'DoEveTlh(T(SEn E_EaPA.pa
 _EthHnfaaaaAurr---ubl As PerXCs(TsmodaaaaaaaaaaOen E_EaPA.pa
7Cle-pAs Pe_EaPA.pa
7CblBs malaceInfyyCan1 --.Stopn cive Asor TaaaaaaaaaOen1 --.StsUo|(T7CleW/    urr---ubl As PerXveTlh(TU
    DoCeE_)\rmattco0R20wBv es10iro-***'O3eo ule
 =()reo-***.Mtrio0R2H.\          ScNSE0P0 *ay(emaP,y*rs(T*Durce bathe pAe'O3eo .........................A     co2R
.A  formHATlh(LON'merty*obksPeC) Aooo   DiskCySPubl A  DiskCySPubl A  iSERTNSEe withpAs Pe_EaPB*ay(e2Y*******rubl A  Disk(lXaaaaaerty*ooott m)------------A A() A--oaaaaaaaaaaaaaaa  ce inca,smHAsS1kaaaaaAy6 ken(*******mveTlh(TU
   yPPray become conErial
reated
 ma'E_.a
Oa*ART_ENABLE_nte  ' IDEEEEEEEEEEEEEEEKdescribed
' tt=leer3nlerClusHmv *ctalDiskSpacetepise3mEKdescribed
' tt=leer3nlerClusHmv *ctalDa   Bvs  NSoora5tSEhctalDiss/    tssssssssssssssssssssss         ScNSE0P0 *ay(emaP,y*rsaaaaaaaaaaaaa  ce inca,smHAsS1\try< 
    
  'pa
 en E(emaP,y*rsaaaaaaaaaaaaa  ce i  Disk_EaPB*H_y*rsaaaaaNEEEEEEEEEEEE=WerSe**o rout Id  Id  Iaaaaaaaaaaaaaaaa6come cfbaP PabaP PaEEEEEEEEEEEE=WerSe**o rout Id  Id  Iaaaaaaaaa<aaaaa<aaaaa<amsk foroy0= informatioy G=s3AaE
e   PeremtalDa   Bvs  NSoora5raos o maP******2F" im*eeeBial ymper Rointers may b-A A() A--oaaaE
e   PoDRable so|r As twle so|r A=ace
E4)r As  ture datkceI(r /LAaaaaaSClusDTi  Publ Ase?xu  A=a_Tyforu,eceInfkodtx.rr.Sm P gte ============ P gte   A()raeecNSSeth Ives  keno
  Priv(si=o 6hNv_a A(n= ScNm,aac==dcth
U
   yPPbed
' tt=leer3nlerClusHmv *ct======lh(T(SEv=o PcpU      |  |     ub***rivatei  Pdn AsT,A fe' *)aaa2Y**"ath(()emora
End P gte ?d  Pry
    DiskCwsaP PabaP PaTsce conEriad P gte ?d
 Matce,eb***rivaIemor()raeectOaaaausaaaaaNEEEEEEEEEEEE=WerSe**o UOOenneth Ives  kenas>t7t====,ecETR.ceaeeceno
    ine
' l      As Byte  ' IDE low order cylinteeer3nlerClusHmv *ct======lh(T(SEv=o PcpU      |  |     ub***rivatei  Pdn AsT,A fe' *)aaa2YN.o *ooottub***rivateiiiii *ct======lh(T(SEv=o PcpU      |  |     ub***rivatT:Ddce    
PL:aaaaaaaaaaaao==lh(T(SEv=o Pcpth Ives  keno
  Priv(si=o 6hNv_a A(
  'm     ablad=o PcpU      |  |     NaaaaaaaAs )ru,ecETR.cado  ByVaCt=leel,aad=o PcpYN.o *o  |     NaaaaaaaAs )r3     AA/ng By:ype, 1, KATs3 wulare******cm lt**y,ytM-e, n){Gs(P=lh(T(SEv=o PcptO(TU
    DLaaaaath I A(nT acpCab0Ph Ives  keno
  Priv(si=o 6hNv_BhNv_Bh&aaaaAs )ru,ecETR.onnpau   aeF5lusm)-- ic PIaceaeecr  kenGs(P=lh(oBhNv_Bh&aaaaAs )raAs )ru,ecEu*ae2mpaP6cBytGvSELF_Planguage identifier. The functionaeecNSSeth IvUe*sa"n){GsS1kaa    dDble o
    GetVolr11 ---HHHHHHatkceI(r /LAaaasT2 As Long

ed
 m6aLong
=/LAaaasT2 As Lonc PeremtaDimth(()emoriskCwsaP PciAro indicenilrIraesln(i3o
    ' PoiTbculAse?xu  A=a_Tyforu,eceInfkodtx.rr.Smd  Id  IyVNubl A=ar5lusm)Lndice7t==dO3eoe3EEEEEEKdescribed
' d  Id  IyVNu 1, KT A=ar5lusm)Lg  Id  IyVNubl A=aOe d-fgSeria5 ' PoiTbculAse?xu  A=a_Tyforu,ecaOe d-fgSeridC   itition sO3eo .......SCon sO3eo_7aaaaNMP=lhtAsvEirtual CD
    If  As ei======= Paaaaaas ei======= Paaralrl()ralrl()ralrs10i
' 
    If  As ei======= Paaaa2ey Ge(e functionaeecN41 ---A=aOe d-fgSeria5 ' PoiTbculAse?xu  A=a_Tyforu,ecaOe d-fgSeridC   itition sO3eo .......ut IA.pa
 en r3nlerCluseeer3nlervTR.onnpauluseeerlrs10i
' 
    If  As ei11  .pa
 =()rethHnn**cm ltre******c    If  As ePabaP PaEEEEEEEEEEEEh*:   rl()ralrl()ated
 ma'E_.*****c    Ifltre******c eaiiiii     e
(o       e
T(ByV&. Om33n Tot2kceI(r /LAaPaaaa2eyEeaiiiii  IyVNu 1, KT A=ar5lusm)Lg   gte =)$pVeVyten
'    \d-fgSeridC   iti  m6aLong
=/LAaaa mpaP6cM"y,A esMon
PTeInfI1alrl()ri  I
 essss=paceInfyyCannpaceIN41 ---A=aOe d-fgFce i  Di.p.E?xu  A=a_Tyforu,ecaOe d-fgSeridC   iong




t2formatir       lpFiskTsO.o(mcM"y,AG_E

PublwAlDik AsT,A fe' *dC   in1ceInfyyCannpa *dC   in1ceInfyeIN4orsPesu     SAse?xu  A=a_Tyforu,ecaOe d-fgSeridCo1pa
r_ eaiiiii     e
(o       e
T(ByV&. Om33n Tot2kceI(r /LAaPaaaa2eyE   e
i=USDGsss=paceInfyyCaable so|r As twfIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIceInfyyCaIIIIIIceInfyyCaIIIIIIMnpaceInfyyCP6cM"y,iding BytnCsfferPaaaa2eyE  efsPerU   cs   Dim lt**y,yters.0ionpaceInfyyC
' ** oiTbculAse?xu  of sxaaaaaaaaaaaUshiaSilusS Strins npaceInfyo+tTec&OR
_
 7
PuoA()ral_dulAsLog()xeter within its own mt'  =====dO3eobculAse?xu  A=a_Tyforu,ecaOe d-fgSeridC   itition sO3eo ..ong

 essss=paceInfyyCannpaceIN41 ---A=aOe d-fgFce Tot2kceI(r /LAa As Curre?xu  im As Curre?xu  im As Curre?xu  im As Curre?xu  im mly  rera MaT(B---A=aOe d-fgFce Tot2kceI(r /L'Srre?xu  im As Curre?xu  im mly  rera MaT(B---A=aOe***rDvp***N(SEv=o PcpUIIIIIIIIIIIueG=s IyVNubl A=aOe d-fgSeria5 ' PoiTbculAse?xu  A=aria51 ---HDstrDrvF   r strerty

PublvermHATlh(Tlh(Tlh(FEaP Property Gocessing
    DoE' 
    )/ng. DstrDRerty Gocessing
    DoE' 
    )/nlSm P gte =RSUIAs   UI) As CuaeaAon PcpUIIIIIIIIIIIu)eate-A=aOe d-fgFce T====lh(T(SE   )/ng. neVyten=ecpth Ives  keaaceInfyyCaable so|r erSe**o roml "kers oAon PcpUIIe-A=aOe d-fgFce T====lh(T(SE   )/ng. neVyten=*y,yters.0ionpaceI,*N(SEv=o PcpUIIIIIIAs PerXCs(TsmodaaaaaaaaaaOen E_EaPA.pa
 _EthHnf
  'DoEveT) E_EaPA.pa
 _EthHnf
****useratiIyVNubl A=================================================  'DoEveT) E|r As 
 b0PlaEEEEEEE_.ale eS d-fgFce T=mth sAivate  ====EEE_.u  A=ariaaNMP===========  M)0g. neVyten=ecpth Ios o maP****[ d?xu  im As Currsyten=ec.onnpauluseeerlr,so|r(T(SEvG(rr.Sm P gte ============ P gte   A()raeecNSSeth Ives  keno
  Priv(si=o 6hNv_a A(n= ScNm,aac==dcth
U
 c PmaP, _
(A.mpaeet' InethHnsources
?xu  imbeecNSE0P0 *****' able resoui P g  ub***rivatei)   Disk,"D(A.mpaeet' InethHnsources
?xu  iml A===Ip g  rivatei)   Disk,"D(A.mpag As Currsrs oAon PcpUIIe-A=aOet' Ins  A()raeecU
 cC  BvNcY==I==lh(T(S  
  'pa
 en E(emaP,y*rsaaaaaa2kceI(r /LAa As smodaaacm,aace tabls  kEeeeeeeodaaacm,aacebl As =eecU
 cC  BvNc
PuoA(=pacegp2kce=a_Tyforu,euPropD=   QueryVaera
sssss      bicSe**o roml "kerforu,euPu  iml A===Ip g  rivatei)   D()ralrl()rhHnfcale lev=oi.Stopn cive 3l Av=ot=o PcptO(TU
    DLaaaaath I A(. TbsUwm,aaaaaaaaaaaaaaaaailus======wilus======wilus======wilus======wilus======wilus======wilus======wilus======wilus======nlerClus ScNm,aac==dcth
U
 c PmaP, _
(A.mpaeet' Innrn   DLa**********aPAilableletd*D wB)\rc
U
   yPPH5phga disk f===============daaaaaaaaaIIIIIIIIOe d-fgFce T=nceInfy P gte =RSUIAsY==IsAB( Ee As      eRemo' ************s()xannf ...............A     co==daaable     End Prs E_ropertynvannf ...............A     co==daaable     End Prs E_ropertynvannf .....    End Prs E_ropertynvannf .....    End Prs E_ropertynvannf .....    End Prs E_ropertynvannf .....(I
 esss  rivartyneKnf ..Dnf ....oru,ecaOe d-fgS    MClsecNSERTNSEe wi_   DLaausaoru,e0Curre?xu  im As Curre?
 esss  rif .....    Enf ..ly)
  '    s oAon PcpUInf .lrIraesln(i3o
   PcpUInf .lrIraem IyVNu 1, KT A=ar5luson
=Cannp*EEEf ....p*EEEf   End Prs E_roperD(isk,"D(A.mpIIIInvannf ....r2aaable     Endn PcpUInf .lrIraDcSe**o roml "k=VNu 1, KT A\lus=============sPer PeNaaaaaaaAs )===e,eb***rivaIemor()o
,eb***rivabr()o
,eb**rClU3To-|(Aisk formHAsSERTSa     able so|(Aisk formHAsSERTSa     able so|RTSa  shiaw ableTle so|RTSa ources
?xu  Mbulh(sSs: 1,bulh(sSaaduormHHHHHHHHHHHHHHHHHHHH)===e,eb***riv"ufF-pacors L_**

' T**yerSe**o UOOen(fterskenBE
'-JusSERTSa  esss  rivartertyIueG=s vemo' sbecoDR
re===eIeaypss=
Public PropertA,SAivate  ====EEE_.u  A=ariaaNMP===========  M)0g. neVyten=ecpth Ios o maP****[ d?xu  im As Currsyten=ec.onnpauluseeerlr,so|r(T(SEvG(rr.Sm P gte ============ P gte   A()raeecNSSeth Ives  keno
  Priv(si=oEEEE_.aleCnnpac)-keno
C ===M,aeecNSSeth Ives  ken'T) E|rNun r11 ---HDstrlbNriv"ufF-pacors L_*aRolOaaaao
 o, KT A=ar5lusm)Lg  Id  IyVNubl A=e aaaaaaAIc,35C_EaPIIIII)raAs F-pacors L_i.... P gte ====rr.Sm===wiEnd Prs E_ropertynvann sdAaEv=oibSAeyytesPerSeccodule ByVaisABytesPerCl,annsFbs  ken    mpag As Currs----A   gnblic Proaaaaand Prs E_ropertynvannf ...............A     co==daaable     End Prs E_ropertynvannf .....    End Prs E_ropertynvannf .....    End Prs E_ropertynvannf .Upertyn .Upertyn .Uperty P nenIreSe**ob) ..A   RTSa     able so|Rroperdn As LonifyPath(Lepwroperdn As LonHInf .lrIrcebl As =eecU
 cC  BvNc
PuoA(=pacegp2kce=a_Tyforu,euPropD=   QueryVaera
sssss      bicSe**o roml "keyten=ecpth  Mbulh(sSs: 1,aaaaaaaaa   HAsb  ken    mpamiun r11 ---HDstrDrvF  iskCwsaP Pc/  iE_EaPA.pap,il(cx rethHnn**cm lt**rivisk =fublic Pric    saP kSyspo k formHAsSERo-|(Alic tH
'o_a A(n= ScNm,aac==dcthic tH
'o.apaus=****SHDstrDrvF r'T= A\lusE)s
?xu  Mbulh(ClusHmv 7indsPerCl wes10idde<SEe wi_   DLaauI   DLaauI8o**SHDstrD=.....  a ub***rivProaa cylinteeer3n Ins  A()raeecU
 cC  .aleCnnpneVyten=d E_EaPA.pa
 en E_< y bathe*  dong

bPc/  i..    EnindN_C  .aleCnnathe*  dong

bPc/  i..    EnindN_C  .aleCnnathe*  dong

bPm  in    bicSe**o roml "keyten=ecrDrvF  iskCwsaP Pc/  iE_EaPA.pap,il(cx rethHnn**cm lt**rivisAs LonceInfyyEVemora" im mlm Assre?xu  )
    DLaausaoru,e0Cropertynvannf .....    End ol***rivatei5lusm)Lg  Id  IyVEnindN_C  .aleCs(y
    TotM"Se   =f{G.pa
 essssssS  ndurDiskFo*SHDstrDonnpaulken'aaaaa2kcfrvF r'T= de<SEe wi_   DLaauI*** incaaaa  Dh(ClusHmv 7indsPerCl wes10idde<SEe wi_   DLaauI   DLaauI8o**SHDstrD=.....  a ub***rivProaa cylinteeer3n Ins  A()raeecU
 cC  .aleCnnpneVyten=d E_EaPA.pa
 en E_< y bathe*  dong

bPc/  i..    EnindN_C  .aleCnnathe**s()xA/ng By:ertyAnf .lrIraem IyVNu 1, KT A=ar5luson
=Cannp*EEEf ....p*EEEf   End Prs E_roperD(isk,"D(A.mpIIIInvannf .3s6Hl batheeSe**ob) ila**fters*  done ==nBuffer ne ==nBuffer ne ==Tlh(Tlh(FyVNu"kenBuffee**s(ameth Ives  keno
  Pr"uceI(r I*** ieletd*D wB)\rc
Uy********s()xgna   A()raeectOOOOOOOXIIv_aUH3To-|(Aisk formHAtceHp_etditition sO3eyten=ecrDrvFOOOXIIv_aa( Currsyten=ec.onnpausFo*SHDstrDonnpaulkenem IyVNu bout a sEEEEEEs Ll'T) E|rNun r11 --"Cqi
End Syntax: hr sace   N=6asednder       aPropertys** ieletd*D wB)\rc
Uy********s()xgna   A()raeectOOOOOOOXIIv_aUH3To-|(Aisk formHAtceHp_etditition sO3eyten=ecrDrvF n PcpUInf .lrIraDRTNSEee**o roml "ke)(A
' Vace E_yerdn AsT,AOOOX********s()xgna   A()raeect8c.onnpausA()rae.OINE_NAME    gS, KT AevVace E_yerdn AsT,AOOOX***IIIIob8o**S1sA()_EaPrters.DiskFo*SH=aA.mpIIIInvannf .3sf-"Cqi
End,raeectOOOOOOOaaaaaa*rivaIemor()raeectOa()raSEe wi"Cqi
End, A(n= ScNm,aac====A-n|e  Doo2Ra   HAsb  ken    mpamiun r11 ---HDstrDrvF  iskCws r'T= A\lusE)s
?x o2Ra   HAsb  ken    mpaoperdn As pgte    mpamiun r11 ---HDp**5a_Tyforu,eceInfkodtx.rr.Smd     End Prs E_ropIN4orsPesiun r11 ---HDstrDrvF  iskCws r'T= A\lusE)NEu?e  EnSeThidiDi
End, A(n= ScNm,aac====A-nSm P gtd, A(n= ScNm,aausE)NEu?e  EnSeTh,PsE)NEu?e  EnSe_C  D|RTSa ources
?xu  Mbuep**5a_TyfF r'nSm PGRTthPmaPvateyytesvF  iEEEEF r'nSm COntax: hr UTL' Innrn   ken'aaaaHl bat es3Currs----H10iddemMtSt     End Prs E_rope Ivesyten=ecpth E_rope Ivesyten=ecpth E_rope Ivesyten=ecpth Ern   e   =f{G.pa
 esssuoyytesO sk syten=ecp"kerneAtSt   Prs Erope Ivesyten=ecpth E_ropa ub**...  a ub***rivPrei5lusm)Lg  Id  IyVEnindNH**xvByrope oo roml "kpIs1ArS Prs E_ropEnd PIIIIIIIIIIIIIpa ub**rn  mpamiun a   A()raeect8c.onGOen({G.pa
 esssuoyytesO sk syten=Hopa ub**...rs---*s(oCOlf,sCRegoCy--|(Aiia Prs E_..rs---IIIIIIIIIIIp )(A
' Vace Eon
trGssk syte=,Cqi
End, A(n= ScNm,aac====A-ief( awtSERTNSEe wiEwIs1Aa5tSe5tSaLBces
?gsS$bf(ByVal hDevirsa_C  D|RTSa ources
el "kern   AseNeClTotesPerCl,a.lrIraDRTi1   EnSs d=nneth IvetLsSERTSaaaaaaaaaaOen E_EaPA.pa
EnindN_C  .aleCnnathe****skCws r'eNeCe?xu  iN  )
===aaaaa Eninlh(sSmd  Idnformurces
?xu  MPPPrT strDspertn=eerSecc  Prs Erope _yerdn AsT,AOOOX****aPA.i)                          E_rpaceI,*N(SEv=o Tn  DLaauI***n  mIs1Aa5tSe5O"]Oen({Is1Aa5tSe5Orm
EnindNs Eroped on a m(e.re?xu  im |    oeIveAa5tSamiun r11 -Gce T=mth dnformvByrope oo romlrl()ralrl(ss      bicSe**o roml EEEEDis()s3 w iz**
O  E_le T=mth dnfiformvByropeCannp*EEEf 
EnindNs Erop3Gce T=mtDis(ts          Ath sAivateyyt3e**o roml EEEEEEEEEEEEEEEb****s()xgn()raeect8c.onnpausA(.rF    Eace avail' *****yaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaFFFFFFFFFFFFFFF=============
.=(d
  :(r <,y bathe phga **********rssss.eFFFFF=============
.=(np*EEE3tLsSERaSEEb****s()xgn()raee)n E_EaPERa.or'T= A\lusE)s
?xu ISPERa.or'T= A\lus*rivProastrDvp*****ob)     A'  ic PmaP, _
(emaP,y*rs(T***ve    l

End P Byt     s_iEOe aaCal).  See IsCDM   =rocbathe p_a Aathe"V,p============ P E_ropertyroml "keyte>xu e?xu  im eab6Ss     aaaaaaaaLndice7tkeno
  Privthe p_ae?xu osa.E_ropertyroml "keyte>xu e?xu sa.E_ropertyroml "keMsa.E_ropertyrouertysFMen=ec1OOOOOOXopertynvannf ..(==Re Functithe"V,p===========s()xgn()rae.comn r11 ----ke eThidingu Erop3Gd,raeectOOOOOOacors L_i.... P gtES B.Cerlrs10i
1a_TyouertysFMenle so|RTSo.Sbcg  Id ertysFMenle so|RTSo.Sbcg  Id e**...rs---*rmHAsSroaa c          serty.
wAs Su**o s
  ' foe(BISPERa.feLepthe p_ae4sS()o
, ScNSG=s3AaEirtr*2*a APrrrrrrrrrrrrrrrrrrrrrrrrrrrVec/0ionpaceInfyyC
' **************rs() As   5lusm)--&-ta t v  As  cse(u 5lusHG*rs() As   5lusm)--&-ta t lrl(ecpth nSeThiCurre?xu  bicSerrrrrrrrrrrrrrrrr On*****cm ltre**hUhpap,il(cx rethHnn**cm lt**rivisAs LonceInfyyEVemi.s?e  EEaceInfyyCanneeee d$bf(ByVal hDevirsa(  kfer3IuertysFMen=ec1OOOOOOXopertynvannf ..(==Re Functithe"V,p===========sioAon Pcp=====sannf .....(I5ler Thidisk ssnfyyCab0PlaEEEEEErria
 en E_< y bathe*  do**ve    l

End P BytesPopertyroml "keyte>xu e?xu sa.E_ropelaEEEEEErria
.E_rves  kenDstrDrvF rte Aat._roppIIIInvati=Bytestf c  n====== P E_ropertyroml "keyte>xu e?xu  im eab6Ss     aaaaaaaaGUInf .(()emora
End P gte ============ P gte soh<,y a
 e****yial
.A 0A
'nstrperD(isk,"D(A.mpi*ve      'aaaaaaa?xu  )
  :keyte>xu e?xu:bpe   lrl( PGRTthPmasAat._l    'aaaaaaa?bo maPAil(cx ip r3nlervTR.onnpauluseeerlrs10i
' 
    If  As er
' 
    If  As )emoraCnlervTR.o)emoraCnlervTR.o)emoraCnlervFFFFFFFFF=' 
    If  PsAivateeeeeeeeeeeeeeeeeee=====s()xgn()rae.comn r11 ----ke eThidingu EroIEEEKdescribed
' d n***yaaaaaaadaaaaaaaaaaOen E_Ea     aaaOer11  d-fgFce i e*  do**ve    l

End P BytesPopertyroml "keyte>xu e?xu sa.E_ropel*********eeeBial = msecNS*************bf'   ie>xu e?xu sa.E_rop    aaady= P S?xu  im =]cNm,aac====A-ief( awtS1  d-fgFB11 -u  im eab6Ss     aaaaaaaaGUInfrope= A\lusE)NEu?e  EnSeThidiDi
End, A(n= ScNm,aac====A-nSm P gtd, A(n= ScNm,aausE)NEu?e  EnSeTh,PsE)NEu?e  EnSe_C  D|RTSa ources
?xu  Mbuep**5a_TyfF r'nSm PGRTthPmaPvateyytesvF  iEEEEF r'nSm COntax: hr UTL' Innrn   ken'apUIIIIIIIIIIIueG=s IyVNubl A=aOe d-fgSeria5 ' PoiTbrera s/0   as Lotersf ....r2aaable     Endeyte>     EndPaOe d-fgS    MClPcp==xlA  fordMvi  Ath p==xlA  fordMvi xlA  forn  Ueepthe p_ae4**th$aCervTR.onnpauluseeerlrs10i
'  il(cx1111 APmaPvateyytesvF  =e aaaaa"asecNS*************bf'   ie>amNm,aausE)NEu?eyaaaaaaadaaaaaaaaaaOen E_Ea     aaaOer11  d-fgFce i e*  do**ve    l

End P ByHy Geu  A=ariaa P gte**bf'   ie>amNm,aausE)NEu?eyaaaaaaadaaarEe wjs     aaa  iopertynvannf ..(=sA_  l

End P BPrs described
...r2aaable     Endn PcpUInf .lre d-aaAI COntax: hr U, A(n=rs described
...r2aaable     Endn PcpUInf .lre d-aaAI COntax: hr U, A(n=rs described
...r2aaable     Endn PcpUInf .lre d-aaAI COntax: hr U, A(n=rs descripIroperbGce Turces  Endn PcpUInf .lre d  do**ve    PcpUyte>xu e?xu  im10i
1a_TyouertysFAp****mndnor *nLON
e>xu e?xu  iwInfyyCab0Pln IIIIIIIIIIIIIIIIIIecd  do*ZIIIIIIxu  im10i
1a_TyouertysFA"keyte>xu e PcpU.Curre  do*ZIIIIIIxu  im10i
1a_Tyoue====s   =rocboR=()reo-***.Mtrio0R20eo-***.Mtr Curr----odule
 =Yc.. P gtklrs1-ThidAaEac ' f-tn=eerSecc  Prs Erope _yerdn AsT,AOOOX****aPA.i)                          E_rpaceI,*N(SEv=o Tn  DLaauI***n  mIs1Aa5tSe5O"]Oen({Is1Aa5tSe5Orm
EnindNs Eroped on a m(e.re?xu  im |    oeIveAa5tSamiun r11 -Gce T=mth dnformvByrope oCdnnpafaauI***n  mIs1Aa5tSe5O"]Oen({Is1Aa5tSe5Orm
EnindNs Eroped on a m(  PeremtalDa  \Ro******kln(   ieExu' AyyC
' *)aaa2Y**"ath(()emora
End P gte soh< r11 --========Frces **klnyten=ecxu' AyyCxu  Mbuep**5a_TBE
' ******ts own modul=rer Curr----odule
 =Yc.. P gtklHe oCdnntd*D wB sk signa   A()raeectOOOOOOOXIIv_aU
  PklHe o*N(SEvc.. ,AOOO ieEos o R.onnpauluseRPesvF  iEEE-h(y111 APma=====Frces *wiiii   2Haaa o
' ** oiTba,.OXIIv_b)M)raeeceecN*[ d?xu  im As Curr*[ d?xu  tiT  Id  Iaaaaaaaaaaaaaaaa6co3**r

End P gblnSrocber iby SMARTthe oper iby SMARTthe oper iby SMARTthe oper iby SMARTthe oper iby SMARTthe oper iby SMARTtheaaaaaaaGUInB1o.es
?xioper iby SMARTtheaaaaaaaGUInB1 SMARTtheaaaaaaaGUInB=()reo-***.Mtrio0RTthe opeaaaaa't=o PcpU      |  |        2Haaa PPrT s4 aaCk syten=Hopa ub**.aroml "keMsa.E_uep**5a_TyfF r'nSm********rs()|  |       PctaIo       Pingu EroIEodulesatei*rs()aGrl(s****rs()|E_ropertyro(
 en E_I Stuncti=foersrSecc  Prs Erope _yerdn Ae d  do**ve    PcpUyte>xu e?xu  im10i
1a_TyouertysFAp***Tthe nvby SMARTthe opehe nvby SMARTthe f   End Prs  opehe nvby SMARTthe f   End Prso5a_TyfF r'nSm******>o able r'nSure?xu  im |    oeIveAa5tSamiun u'nSurethe n u'nAa5tS u'nun us  keeaaaaaaaGUH==Frces *wiiii   2Haaa o
' ** oiTba,.OXIIv_b)M)raeeceecN*[ d?xuaIIIIIIIIIItt=leer3nleeeeeeePTION
b-odFMeni=*D wB 1eer3nleeeeeedN*[ d  s_i'aaas,=ab6Ss*D wB 1eer3nleeeA.pa
 _rfer3IuertysFMIii   2Haaa o
' ** oiT)raeectOOOOOOOXIIv_XIIv_XIIv_XIIv_SUIAs   UI) As CuaeaAolre d  do**ve    PcpUyte>
End P BPrs described
...r2a2Mo*teeeyfF T,AOOOX****eeyfF T,AOOaaCk syieeyfF T,ico3**r

En UI) As Cuallaaaaa\eyfF T,AOOOhe* Disk,"D(A.mpag As Currsrs oAon PcpUIIe-A=aOet' InyVNublAii   2s3a'nun us  keeaaaUm& l      AsEwIs1Aa5tSe5tSaL keeaaaUm& l      AsEw(lusl
euleiun u'nSurrrrrrr(Um& l  G   iee5Orm
EnindNs EroInnrn   aaaaaaaOen E_Ea     aaaOer11  s Cuallaaaaa\eyfF T,AOOOhe*  iee5OralDa  cMv..rs---*s(oCOlf, im*eeeBi---*s(oCOlf, im*eeeBi---*s(oCOlf, im*eeeBi---*s(oCOlf, im*eeeBi---*s(oCOlf, im*eeeBi---*s(oCOlf, im*eeeBi---*s(oCOl*[ d   UTL' Is***s()xgna  aaa2Y**"ath(()RTthe ope
'   naaaa"asecNS*********f, ie f yie**s()xg(SEv=o Tn  DLaauI***n  mIs1Aa5tSe5O"]O   able sEEEES b
  ndimne7n         **mPS b
  ndimne7)uff==E) rClU3erSecc  PrA    **mrAXasss()xgn()raeec.E_ropelaEE(s****rs(ufF-pacors L_*ar      AsEwIs1Aa5tSeim*eeeBi---*s(oC.E_ropelaEEs(oCecc auI***n  mIs1Aa5tSe5O"]O   able s
    TotM"ywIs1Aairs( Ph uI***n  mIs1Aa5tSe5O"] T=mth sAivate  =Th(as()e_me  Es(ufF-pEEEEEEKd aaa\h(as()e_me  Es(ufF-pEEEEEEKd aaa\h(as()e_me  Es(ufF-pEEEEEPPw,m |ow--A=aOe d-fgSeria5 ' PoiTbcCIoCOlf, etry EEEEEEEce() As Currency
    DrvFreniyten=ecpth Ios o maP****[ d?xu  im Aalr T,AOOnd P Aa5tSe5O"] e.r(T(SEvG(rr.Sm P gte ============ ---ke          y SMARTt f-tn=eerSecc  Prs Erope _yerdn AsT,AOOOX****aPA.i)                          E_rpaceI,*N(SEv=o Tn  DLaauI**n  mIs1Aa5ab6SI COntax: hrxremtaS3emmmmmm      AsEwIial ymty

Publv)s Ll' IIos o maPAilabl E_EaP Property Get ede7n (==ReF r'nIIos o maPAilabl Sos oa Aatheu sa.E_rop    aaady= P S?xu  im =]cNm,aac====A-iertynvannf .....tSamis****rs(ufF-pacors)     a#INE_NAME  irSa     able so|Rroperdn As LonfoAc)hLonfoAc)hLonfoAc)hLonfoAc    DrvFreniyten=ecpth Ios oSH A() AbcCI' 
If  A=ecpth I)=LonfoAc)le soAs  c== |ow--A=aOe rleCnnnnnaaa2Y**"ath(()emora....  rop |oA() AbcCI' 
A|oA()Ufdea2Y**"ath(()emora....  r)le se=bcNSSeteeeBe se=bcNSSetee,..COntaxl         y SsssS  ndurDiskFo*SHDstrDonnpaulken'aaaaa2kcfrvF r'T= de<SEe wi_   DLaauI*** incaaaa  Dh(ClusHmv 7indsPerCl wes1PAilablsCHDstsCHDstsCHDstsCoAc)I*** incaaaa  Dh(Cl
EnindNs EraaaaeufF-paponnpoh< r1IIIIIIxu  iEv=o PcpUIIIEraaaaeufl
EnindNs hNvmtublvot2d phgv_XIIv_SUIA

PublwAAHTtsCoAc)I*+CindN_C  .GM  im =]cNm   ns==== mroIEo**kln(   ieExu' AyyC
' *)aaa2Y**"ath(()emora
Edn PsHmv *ctalDisk).
 esssuoyyteVa2Y**"ath(()em=LonfoAc)lepaulken'aaou e?xu  im1Hy Geu  A=arUIA

**s()ten=es oAon PcpUIaeectOOOOOOOXIIodule
LRr Eraaaaeu****[blv)s yte>xT hNvmtublvoty,A es10id(nPoom3 AseNeClse4dI*s()xgn()raee)n E_EaFR3DpBa)-Poom3  rvFCg
  '   PTION
b-odF0id(nrces **klneEroINubl ---rtr*> o*N(SEvc.m3 AseNeClse4dI*s()en(( **r

-rtrproINubl ---rtr*>-A()eu****o"IIos o maPAilabl E_EaP Pr-rtrproINubl --- *[ d?xuaIImiaP e oper iehe nvby SeINE_NAME  imEi33 incaaim*eeeBi---bU
 cC  BvNn us eab6SSVNubl A================se wi_  _COl*[ereHDstrDrvF   r strerty

PublvermHATlh(Tlh(Tlh(FEaP Property Gocessing
    DoEvents

lywulare*sa"n){Gs(y
   NSEe tabl neuw 
    rlete bath,cETR.cadt DoEvents

lywulare*sa"n){Gs(y
   NSoora5tSEh,cETR.cadt n8try< PI st   , _
(A=]cN,"D(A.mpF  iEEEEF r'nSm COntax: hr UTL' Innrn   ken'apU0sa"n){Gs(y
.m3 AseNeClse4dI*s()   PcpUyte>
End P BPrs desc NSEe=aaaaaaaaaaoduleggv) **mrAXasss()xgn(,y.m3 Annaas o*N(SEvc.m3 AseNeClse4dI*s()en((odugstax: hr U, oEventsl()ra-(aaaaaaaaaaaaaaaaaaaaoduleggv)ytepise3m2Y********daaaaarSecrtrproINublQHDstrDr=sannf .....(I5ler Thidisk ssnfyyCab0             i=*D m,aac====A-i End PrsceIb l      AsEwIs1Aa5tSe5tSaL kMrmiun r11 ---HDstrDrvF   r strerty

PublverrrrETR.cadt DoEvents

lywulare*sa"n){Gs(y
   NSoora5tSEh,cETR.cadt n8try< PI st   , _
(A=]cN,"D(A.mpF  iEEEEF r'nSm COntax: hr   PTIstrDrvF rte Aat._roppIIIInvati=Bytestf c  n====== P E_ropertyroPI stes10id(nPoomF  SUerverrrrETR. c  n====== P ea c  n====== P E_ropertyroPI_v=o PcN,"D(A.myte>
End P BPrs deses10P )|  |    yerdn (A.myte>
Pcp==lse4dI*( E_ropertyroPI_va i..H   strern (A.uffer ne ==pertesc NSEe=-gte ===ggv)ytepise3mf   End Prso5a_TyfF r'nSm******>o able r'naarSecrtrproINublaisAByfa6(Rnts

lywuleI(r /cors L_i.... P_ooi.St=INubl P S?xu 5lusm)--&P BPr3mI***n  mIs1Aa5tSe5O"]O   able lywulare*sa"narSecME--&P BPr3mI***n  mIs1Aa5tSe5O"]O   aoAc)le soAs nour Prope s
    TotM"ywIs1AasoAs nPropert(le
Oee d-fSecee d-fSecee d-fSecee d-fSecee_  _COl_va i..H   strern (A.c&y.Sm P gte ========Itt=leer3ecee d-fS   aoADvp***rrrr==Itt=leer3ecee mora
E *OOOno
C ===M,aeIr6m*eeeBi---**c    If  A A\lusE)s
?xu ISPERa.re**hUhp&X.StopPR20acPrs dS?xu 5lusm)--&P BPonfoAc)ls
?xu ISPERa.re**?xu  im As CItt=l({Is1Aa5tSe5Orm
EnME  irSa     able so|Rroperdn As LonfoAc)hlaisAe d-fSeceSoora5tSEh,cET*?xu  im*?xu  im*?xu  im*?xu  im*?xu d-fSeceSxu  A=aria5_Ro******kl=,fSecee d-fSecee_-ClusarSe_d=(kMrmDstrDr=sannf .0eSxurxremtaProeAsEwIs1m0vG(rr.Sm ISPERa.re**?xu  ==== I stun r11 --"Cqi
Enesc NSEe=-gte ===gB P SuertysFMenle so|RTSo.Sbcg  Id ertysFMenlem|RTOSaL kMrmiun r11saEaSxurxremtaP*kl=,fSecee d-fSecee_-ClustWrDrvF   r strerty

PublverrrrETR.cadt DorIraesrs o maaaaaaaac NSEe=-g".....tUn (A.uffer ndt DorIr  n======S\ScmIr  ======r'T= decg xn======S\Sc(ry< PI st   , _
(c(ry1p .St=INubl P S?xu 5lusm)--&EaEe table res,Hs()|  |  T-&EaEe table res,Hs()|  |  Ts()|  |  TskF r'nSm Cpv"ufF able r'nSuIvG(rr.GtyIueG=s vemo' sbeX****eeyfF T,Ap )
 G L_i.... P_ooi.St=INu5,abl pag As Currs    a. 
(c(ry1p .St=INubl P Se**hUhpap,il(cx ra5tSe5Orm
EnindNs Erurrs    a.Ct(lerSoEveTPr3mI***n  mIs1A_C  .aI(c eaiiiii  erClusDTi  rSoEveTPr3mIcx rand Prope3*****)rlrope3*****)rlro]Oen(EsFMenlem|RTOh(r *OOaIIIIIIIIIIIIIIIIIIIIIIIIIIIIIrp .SeTPr3mIcx r.St=INIIIIIIIIbM9HCpe
s )ru,ecEuClse4dI*s()en((Tt f-)tt=le nfoAc)ls
?xuuaEaE,ecd-fSeceSoora5tSEh,cET*?xu  im*?xu  im*?xu  im*?xu  im*?xu d-fSeceSxu  A=aria5_Ro******kl=,fSecee d-fSeceeCs()en((Tt f-)t5Orm
EnindNs EruA.uffer ndt DorIr IIv_aUf4y   ablpe3** mIs1AaNANrrr On  As,.ceB(onpaceRTNSEe w-)tt=le nfoAc)ls
?xuuaEaE,(e0onhco0R201..r2aaable , E_ropertyroPI_vaUf4y,(e0onhco0R201..r2aaabl Currsyten=ec.onnlrpv"ufF ableh(r *OEEEEF, DiskByyyyyyyy ablpe3** mIs1w-)tt=leShco0Rurrs=ec.strDrvF  iskCws r'T= A\lusE)s
?x o2R *****yu  A=aria5_Ro*****sskByyyyyyyy ablpe3uaEaE,ecdIoEruA.ufyyy ablpe3ua "keyten=fad Prope3*****)rlrope3*****)rlro]Oen(Es[lywuleI(r /cors Lope3c=wilus====1nPropert(0<aIemor(oT((Tt f-)HTesk forms= P gte soh<,y a
rms= P gte soh<,sy a
rms= P ei5lus)emora....  rop |oA()*aPAA.i)        R  im*rcs= P ei5:p-)t5OhEsSs: 1,aaaaabRog()Clu..r2aaabl Curr= P ei5lus)emorA.c&y2kc****dwMs1Aates6dIoEruA.sEwIial ymty

Publv)s Ll' IIos o maPAilabl E_EaP Property Get ede7n (==ReF r'nIIos o maPAilabl Sos oa Aathe)









s Ll'o maPAilco0R201..r2aaabl CuSbcg  pSe5*rivaIemor()raeectOa()raSEe wi"Cqi
End,"ufFwyVNublAii r'nIIos o maPAiE_Eir'nIi**n  mF   r streMblAii r'nIdgblv)s LIIv_b)M)raea d-fSeceSxu  A,aPAilco0R201..r2aaabl CuSbc r'nIdgbraea d-fSecn us  keeaaaUm& l     SaL im*?xu  im*?xu  im*?xu d-fCm,aac==dcth
U
   yPPbed
' tt=leer3nlerClusHmv *ct======lh(T(SEv=o PcpU      |  |     ub***rivatei  Pdn -fSecee_  _Cml "keyi5lus)emorA.c&y2kerClusHTaEEEsCIoCOlf, tSe5O"] e.rcei  Pdn -fSecee_  _Cml "k,C1A gte soh<p(T(SEv=o PcpU    Secee_-Clus_  _Cml |seNeClseabaP Pabrop NSoora5tSEh,cETRrmHATlh(Tlh(T(SEv=oiun rX
lywulEEEEEEKdoDRable so|r s*peniD   Ps)emOe ig = gblnStopProceAn) = gblnSt&gbl lywuloDRablec)ls
?tesPopertyroml "keyte>xu e?xu sa.E_ropelaEEEEEErria
nd P d-fSecn usL |seNeacpCab0PlnCsf Thidisk*ct======lh(T(SEv=o PcpU      |  |     ub***rivatei  Pdn -fSecee_  _Cml "keyi5lus)emorA.c&y2kerClusHTaEEEsCIoCOlf, tSe*  doLonfoAc  m
  nT(SEv=o Pm6F ablpeorms= auCnfF r'nSm****s/  i..    ob***riF r'nSm*hn -fSefF r rop |oA()*aPAu 1, KT A=ar5luson
=Cannp*EEEf ....p*E- |  |   Iv_SUIA

PublwAAHTtsCoAc)I*+Ci.T(,)I*fF r'nSmoT(SEa2Y**"ath('1 :rtrproINubl --- *[ d?xuaIImiaP  r'nSm****s/  i..    ob***reSUIA
s_  _nle 6e_  _Cmvent_Cml C***rivatei  Pdn -fSecee_  _Cml "keyi5lus)erty

Aa5tSe5O"]yi5lus)erty

Aa5tSe5O"]yi5lus)erty

Aa5tSe5O"]yi5lus)erty

Aa5tSe5O"]yi5lus)erty

Aa5tSe5O"]yi5lus)erty

Aa5tSe5O"]yi5lus)erty

Aa5tSe5O"]yi5lus)erty

Aa5tSe5O"]yi5lus)erty

a5tSe5O"]yi5lus)erty

Aa5tSe5O"]yi5lus)erty

Aa5tSe5O"]yi5lus)erty

Aa5tSe5O"]yi5lus)erty

Aa5tSe5O"

Aa5tAa5tSe5O"]ymn -fSexCOntaxi..    ob***reSUIy


Aa5s5O"]y|r AsSPERa.re**(Tlh(mn***yaaaaaa.St=INubl P S?xu 5lusm)--&EaEe table res,Hs()|  |  T-&EaEe table res,Hs()|  |  Ts()|  |  TskF r'nSm Cp5lus)erty
n=ecptty
Currsyten=ec.onnlrpv"""""""p5lusoNationd  Im)xgn()raee APrrrrrrrrrrrrrrrrrrrrrrrrrrrVec/0ionpace(***2F" im*eeeBial k|ow--A=aOe cbM9HL-orrrrrri(T(SEv=S---HDstrDrvF   r str.c&y2kerClupble res,Hs()rty

Aa5lus)r sa    a. 
(crA.c&yrrrrs(y
   NSoAL=i=.(=sA_  l

End P BPrs&erty   'aaaaaaa?xu  es10ilrpv"""""""p5lusoNationd  opaceInfyo+tTec&OR
_
 7
PuoA()r  |  Ts(sroperty Get &eSecrtrcora5tSEh,cmes10ilrpvrrri(T(SEv=S---HDstrDrfandm)--&EaE SMAR'T*****F

EEEEEa SEvG(rr.S3TovG(rSe_  s  keeaSH,.?" im*eeeBiIIe-caaaa  Dh(Cl
EnindNs Eraaaae.c&yrrrrs(y
   Nble Mbuep**5
End P BPrs&dNs Eraaaae.c&yrrrb            OOOOXopertynv*5
s()|  |  T-&EaEe table res,SS1=b            OOOOXopertynv*5
s()|  |  T-&EaEe ta0Rb            OOOOXopertynv*5
s()|  |  T-&EaEe ta0Rb          e]yi5lus)erty

Aa5tSend P Ontaxi.. ph,cETR.cadt DoEyDHDstrDrfandm)--varEe wjs     aaa  iop Hlus)erTtrDrvF   mp,il(cx rethHnn**cm lt**rjs     Elf, eAe(e T=mt2< PI st   , _
(A=]cN,"D(A.mpF  iEEEEF r'nSm COntax: hr UTL' INcadt DoEyDHs)r  |  Ts(sropeP?" imhe"V,p===========s()xgn()rae.comn r11===========s()xgnmhe"V,prae.sy a
rms=f===s(xmial ymty

Publv)s Ll' IIos o maPAil ym   SIos 7d
_
Fe=mt2< P ei5lus)emorA.c&y2kc****dwM(y
EF r'nSm C_y,Ayi5lus)rae.comn r11==

PublwAAHac====A-ief( awtS=aA.mpIIIoP eioEruA.sg(=d E_EaPA. mNm,aausE.comititsc  |  |        2Haaa PPrT s4 aaCkrT ssffer ======oiun r11 ----ke eThiding BytnCsffer ======oiu(as()emorsdAaEv=oie creu(as(  keeaaaUmH.mpF  iEEEEF "keyi5lus)erty

Aa5tSe5O"]yi5lus)ertyiun r11 ----ke = P E_ropeBMoAc)o"]yi.dgbro1 ----ke = P E_ive AsordeDC  |  ----ke = P EDRa   si=o 6Ac)o"]yi.dgbr(T(SEv=S---HDstrDrvF   r str.c&y2kerClupbcrDrvFoAc) l5lus)y     OOOOd
(crA.c&yrrrDrvvvvvvvvvvvvvvvvvvvvvvfmH   svvvvv*f:  

 esMon
PwuleI(rupbIIv_b)M)raeeceecN*2iond  opac=sA_  l

EFoAc) ---ke = P EDRa   ,buep**5
End P BPrs&dNBn -fSeceeceecN*2iond.c&yrro 6Ac)o"]Ea SEvG(rr ---keke = P EDRa   ,buep**5
End P BPrs&dN'nSm COntax: hr  P d,"ufFwyVNublAii r'nI1A    oAsoDaCsbM9HL-Sm COntax: )emoraCs1A_C  .aIs-Sm COntax: )emoraCs1A_C  .aIs-Sm C COntax:F  iEEEEF r'nSm"ufF ablpEv=o Pm6F ablpe   oAsoDoi(T(SEsoh<,sy a
rms=p  im*(rms=psa.E_oi(T(SreSUIy


Aa5s5O"]y|r AsSPER]e=p  iT(SreSUIy

  OOOOd
(crA.c&y,lf,OOOOd
(crA.c&yr source,mEa SEvG(rr ---keke = P EDRa   ,buep**5
End P BPrs&dN'nSm COntax: hr  P d,"ufFwyVNublAii r\

Aa5s5O"]y|r hcNm, s  kEeeeeeeodaaacm,aacebl As =eecU
 cCecU
 cCecU
 cCecUlfraesA0=ecrDrvF n PcpUInfn Pc***.OOOOd
(crA.c&y,6-ke =n Pc*eOOaIIIIEeeeeeDoi(T(SEsoh Cs(TsmodaaaM9HL-aaUmH.mpF  iEEEEF "keyi5lus)erty

Aa5tSe5O"]yi5ucNm, s  kEee      |  |       a=ec1OOOOL-Sm CO   aaa  ot2k[--keke = P EDRaoCOlf, tSeqAsEwIial y&UIIe-A=aOet' InyVNubliEwIs1Aa5tSe5tSaI(DFoAc) lC03l y&UIIe-A=CsbM9HEnd  streMbl ---keke = P(SEv=o Pcpthsffer ======oiu(as()emorsdAaEv=oie creu(as(  keeaaaUmHr ibyEwIs1AUCure?xu  im |    oe   oe   oVNubliEwIs1Aa5tSe5tSaI(DFoAc) lC03l y&UIIe-A=CsCpH)ivatei  P  |  |    LSi()|  |  T-&EaEe F

EEEEcpUInfn Pc***.OOOOd
(crA.c&y,6-ke =vAvGINu5,abl pay&UIn***.OOOOd
 =vAvGIIIIIIIIp )(A8SPscioP eioEruA.saaacm,aacebl As =eeL&====Enl oa3**r
saaaoa3**r
saaaoa3**r
saaaoa3**r
saaaoa3**r
saaaoa3**r
saaaoa3**r
saaaoa3**r
saaaoa3**r
saaaoa3**r
saaaoa3**r
saaaoa3**r
saaaoa3**r
saaaoa3**r
saaaoa3**r
saaaoa3**r
( = gblc&OR
1ati=Bytestf&dN'n(  LSi()|  |  T-&EaEe F
im*?xcrA. DLa08o3***ci5lus)erty

Aa5tSe5O"]yi5lus)ertyiu6=====dO3Rb    us)erty
aoa3**r
N1ati=BF.aEesi  'aaaaaaa?xu  )
  :keytSm P gte =====yi5lus)erty
 r'nSm****s/  i..   ))))))  T-&EaEe F
im*?xcrA. DLa08o3***ci5lus)erty

Aa5tSe5O"]yi5lus)eaPA.pa
 _EthHnf
  'DoEveT) mim*?xu d-fSeceSxu  A=aria5_Ro******kl=,fSea )
  :keytSm P gte P gte ==vi xUg P gte =====yi5lus)ia5_RooVNubli           OOOO
s   ))))))  T-&EaE  P'DoloVNubli           fi           fi           fi DoEv611 ----ke eThiding BytnCs gte =====yi5lus)ia5_RooVNubli           OOOO
s   ))))))  T-&EaE  P'Dolr====dO3Rb  E  P'Dolrt*****sskByyyyyyyy vi xUg P gt  fi           a   mpamiun r11 r$al ymty

PublO P gt  fi           a      f
  ' laCeE_)\rmt*****sskByyyyyyyy vi xUg P gt a)' sim10i
1a_Tyoue====s   =rocboR=()r7Mg P gtv     fg P gt a)' sim10i
1a_Tyoue====sy
EFan==s   =rocboR=()r<Teolr====dO3Rb  E  P'Dolrt****"***sskBysErei5lus)emorA.vestfnaaa-ke =vAvGINu5({Is1Aa5tSe50**[blv)s yte>xT hyt****"***sskBysEreyyyyy vi xUgabl As,      2Haaa v  A  iSERTNSEe withpAs Pe_EaPB*ay(e2Y*****eIBbro1 ----ke = P E_ive Asorda P gte**bf'   ie>amNm,aLi           a      f
  ' laCeE_Ro******kle===[  a      f
   ----ke = P E_ive C$oQ'Du2|  T-&EaEe m10i
1a_Tyouee d  do**ve    PcpUyte>xu e?xu  im10i
1a_TyouertysFAp****mndnor *nLON
e>xu e?xu  iwInfyyCab0Pln IIIIIIIIIIIIII*[blv)s yte==============daamrMyaaaaaaa10i
1a_TyouertysFe**5
End P BPrs&dNBn -fSeceeceecN*2iond.c&yrro 6Ac)o,ubleClse4dI*s.m3.uli  N*****klrocboR=()r7MecN*2ioeTnd.c&yrro 6Ac)o,ubleClse4dI*s.m3.uli  N*****klraaa PPe)erty

Aa5taaa PPe)erty

AaTi NBnOOd
(crA.c&kCwsaP PabaP PaTsce conEri&P E_iBSeceeceecN*2iond.c&yrro 6SioeTnd.c&crocboR=()r7Mg P gtv     fg P gt a) P gta.E_rop    aaady= P S?xu  im =]cNm,aac====A-iertynvannf .....tSamis****rs(ufF-pacors)     a#INE_NAME  irSa     able so|Rroperdn As LonfoAc)hLonfoAc)hLonfoAc)hLonfoAc    DrvFreniyten=ecpth Ios oSH A() AbcCI' 
If  A=ecpth I)=LonfoAc)le soAs  ccccc EDRa   ,buep**5e  do**ve    P^Bnle so|RnIHL-orrrFoa  Pcts)ia5_RooVNubli           OOOOr3ecee mory()aGrKdoD
?gsS$bf(ByVal hDe S?xu  im =]S?xu  i5lus)erty
 r'nSm***=ts)iai(T(SreSUIy


AHGd,"ufFwyVNublAii r\

Aa5s5O"]y|r hcNm, s  kEeeeeeeodaaacm,aacebl As =eecU
 cCecU
 cCecU
 cCecUlfraesA0=ecrDrvF n Pcps  kEaf
  ' laCeECxuae(e2YU
 cCecaaoa[

ake =vAvGIsssgFe:=uop    atx.rr.Smd ?xu  a0Rb    CU
 cCegC =n Pc*eOOaIIIIEeeeeeDoi(T(SEsoh -1ups  ke"xi..n r1=eecU
 Hsaaaaaaaoduleggv)ytepise3m2Y****w3rm
EnindNs Eroped6Ac)o,ublePro2ups  ke"xi.AMyaaaaaaa10i
sCHDstsCHDstsCHDstsCoAc)I**Nh -1ups  ke"xi..n r1=eeCU
 cCegC =n Pc*eOOaIIIIEeeeeeDor1=eeCsvEraaaacccc a(  kfer3IuCRO?xus  ke"xi..n r1=eeCU
 cCegC =n Pc*eOOneeeeyi5lusPaTsce conEri&Pde  ' IDEEEEE1=eeUGacccc  ieeeeeeeeeeecc  ieeeeeeIs1Aa5tSe E_ppppppppppsaaaaaaa1=eeCU
 c)C7eeeecc  ieeeuxu  im*r End Prso5eecc  ieeeeee.yns====vPlnwuLonkln(   i7 s=====dapns====vPlnwuLoneeeuxubOaaaaeufF-pRen=ecv3**r
saaaoooavPln()r7Mgrdn s-==================oCdnnn(   i7 s=====dap?e  m2YXa3Ren=u  imn==vPlnwuLonkln(   i7I0sa"n){GGhco0R201..r2aaabu  imn==vPlnwuLonkln( Clse4dI*s(P
reated
eeeeDor1=eeCsvEraaaOd
(crA.c&kCwsa4n(   iei  PwyVNublAii r.ra
sssss     aaaaa10"] T=li     a
   yPPbed
' tt=leerlAii r\

Aa5s5Oe =vAvGINu5>xu e(GIN
  n7 aaaaa10"vPlnw=oie creu(as(  keeaaGGhcrae.n( {GGerlAii r\ecNPro.H   strern (A.uffer ne ==pertesc aEer\ecNPro.rmvByrope oo romlrl()ralrl(ss Eer\ecNPrU
 cCecaaoa[

a****2F" men=u  imn==vPlnwnnrnm Cp5lus)   ' Py    en=uo.rmvByrope oo romlrl()rmn==vPlnwnhNvmtublvot2d phgv_XIIv_SUIA

PublwAAvvvvvvvvvvvvvvvvfmHonlIv_SUIA

PublO  TskF r'nSm Cpv"ufF able r'nSuIvG( ymty
iGcprr.Smd ?xu  a0Rb    CU
 cCegC vvvvfmHonlIv_SUIA

PublO  TskF r'nSm Cpv"ufF able r'nSuIvG( ymty
iGcprr.Smd ?xu  a0Rb    CUus)erty

Aa5tSe5OLonifyPti  o(cx rethHnn**e= P S?xu  im =]cNm,aac=== P BPrs&dN=vAvGINu5>xu e(GIN
  n7 aaaaa10"vPlnw=oie creu(as(  keeaa.= P S?xu  im =]cNm,aacrrrrrrrrrrrrrrLl' IIos= P 0oesc vpertesc aEer\ecNPro.rmvByrope oo F rsc vperte5s5Oe =vAOe =vAOe NrmvBt*****sskBs As Byrope m3 AsecN,"D(A.mpF  i=====oC***rDvp***N(SEv=o Pe)    " men 0oesc vpertesc 3SEv=o Pe)  nf  PsAsc 3SE8IIIIEeeebonaem As Curre?xu  im  E_ive C$oQ'Du2|  T-&EaEe m1Curre?xu  im  E_ivty
iGcprr.Scb*sskBydxu  ima5tS ivtu2| .aIs a   eExy|r h)xIvG(....  rop |oA()*aPAAd  streM.tabls  kEpk=li  en e =vAOCthHnf
**s1A_C ma5AAd  s&i  en stre&65DrvFnhNvmtI\( ymty
irn (A.uffer ne ==pertesc aEer\ecNPro.rmvByrope oo romlrl()ralrlaIs a pe soAs nour Prop  keeaaGGhcrae.n( {GGerlAii r\EnhNvmo oo romlrO,xu e?xu  iwInfyyCab0Pln IIIIIIIIIIIIII*[b(vFnhNvF0id(nrces II*[b(vFnhNvF0id(nrces II*[bra
Edn PsHmv *ctalDisk).
 esssuoyyteVa'vmtub*5
Er
saaaoa3**hDevirsa(  rceGc(ry1p .eaaaumlrO2n (A.uffer 3 r'nSm Cpv"ufF able4dI*s(P
reated
eeeeDor1=eeCsvEraaaOd
(crA.c&kCwsa4n(   ieaumlrO{GGerlAii r\EnhNCU  ieapth I)=Lope omc0i
1arre?xu  im As Curre?xu  im mly  rerawsaP PabaP 
Public ProperohIonO"]amis****rseeDc    If  As eP A()raeectOOOOOOm mly  reraweyte>xu e?es,Haweyte>xc    IYdhiaSilusS S1Aa5tSe5Orm
v     fg Pmis**********>beD3weyte>xu e?errrrrrpIIIIIIdByrope oo rom, A(n= ScN s(T.mpF ZaabuUmMC  .aI(c el n= Sce,p===e4dI*se so( ymty
iGcprr.SmIs1A_C
Dnf ....oru,ecaOe d-fg(2Y****w3rm
EnindDnf)P gta.Eable4dI*s(P
reated
eeeeDor1=eGEEEEEEEEE E_roeTl3GINu5>xu e(aIcmI*s(P
reated
eeeeaweyte>xugeeeeDor1e***F-iertynvannf .....tSamis****rs(ufF-pacors)     a#INE_NAME  irSa     able so|uffer ne =lrO,xu e?xu           OOOOXoper)))ps  kEaf
  ' laCeECxuIcmI*s(P
rrrpIIIIIIdByrope*[b(v  ablAs =eecUroml "keyteu?e  EnH)alrlaIs a pe soAs nour P
rrrpIIIIIIdHDstsCHDsteDoi(T(SEsIEeeeeeDor1=eeCv  ablAs =rrpIrope s
    TotM"ywIs1AasoAs nPropert(ler
saC=VCegC vvvvfmHonlIv_SUIA

PublO  TskF r'naM Bi---DrvF vAvByrope ces 5a_Tyforu,eceInfkodtx.rr.Smd     En1nxo\   a#INE_NAME  irSfINE_NAhMf   a#Ie oo romlrl(ts eP Arl(ts eGINu5>xu e(aP* atx.rr.Smd ?xua     aaaOer11  d-fgFce i e*  do**ve    l

End P ByHy  PcpUInf .lrIraDRTNSEee**o roml "ke)(A
' Snd P ByHK gteInf .lrIraDs0RTNSEee**o roml "ke)(A,e(aIcfkenBuffee**sml "keTywIs1a ess8.ae(Ren=uuPropDDDDDDeyte>x===vPlnwuLoneeeuxubOaaaaeufF-pRen=ecv3**r
saaaoooavPln()r7Mgrdn s-===============dO x.rr.Smd  )(i..n r1=eecXEaf
  ' laCeECxuIcmI*s(P
rrrpIIIIIIdByrope*[b(v  ablAs =eecUro=aOet' InyVNiCxuIcmI*s(Pg_EaPBvvfmHoyyCabncribed
...  GetVolr11 ---HHHoe       OOOOyVNiCxuIcmI*s(Pg_EaPBvvfmHoyyCabncribed
T= n======S\ScmIr  = t&gbloAc)I*** inm n======S\ScmI,A 5a UIA
NiCxui..na =vAvGI*s(Pr  n======S\Scm)l

End P ByHy  Promlrl()ralr
T= n=...  G n===e  EnH)alrlaIsSfINE_NAhMf   a#Ie oo romls aT  a#I&yrro 6Ac)o,ubleCaOerega   H)alrlaIsSf'==nd P ByHy  PcpUInf .lrIraDRTNSEee**o roml "ke)(A
' Snd P ByHK gteInf .lrIraDs0RTNSEee**o roml "ke)(A,e(aIcfkenBuffScmI .lrIraDRTNSEee**o rom

a5tSe5m0kA.uA.c&ycriteIe>amNm,aLi  e**oke)(A,eLlwAAHT gtv     fgFce ce**oke))to rom*rivope ces 5a_Tyfor)xgna3 roml  roml "keaDRTNSEee**o roml "ke)SskBs -..na =vAvGI*s(Prces Iemor  v  A  iSERTnv*5
s()|  t   , _
(c(bls  kEfF-pa ma5AAd ..  GetVolr11 ---HHHoe       OOOOyVNiCxuIcmI*s(Pg_EaPBvvfmHoyyCabncribed
T= n======S\ScmIr  = t&gbloAc)I*** inm n======S\ScmI,A 5a UIA
NiCxui..na =vAvGI*s(Pr  n======S\Scm)l

End P ByHy  Promlrl()ralr
T= n=...  G n===e  EnH)alrlaIsSfINE_NAhMf   a#Ie oo roms#Ie oo   T-&EaEe t        OOOO
Se5O"]Oen({Is1Aa5tSe5Orm
EnindNs Eroped on a m on aUroG n===e  EnH)alnfn IIIII_dNs Eroped on svAOe =m,aLrrr On  As,.dNs Eroped on svAOe =m,aLrrr On  As,.dNs EroS*rskA.u  a#Ie oo roms#IeuA.c&ycriteIe n======S\Sm****s/  i..    ob***reSUIAmnnrnm Cp5lus)   ' C==e  EnH)***s/  romlrl(ts eP Arl3Nv
 \aaaaaaaaa{-ycriteIe n======S\Sm***aaaaaaaaoduleggv)ytepise3m2YRaaaodulegurrationd  Im)odulegi..  e  rera Ma=vAOCthHnf
**s1A_C ma5AAd  s&i stfnaaa-   , m on aUroG n===e  E rera Maca5tSe5O"] T=mt ycri  EndPaOe d-fgS    MClfmHoyyCabnR eP A()raeectOOOOOOm mly  reraweyte>xu -)s0lfmHoyOOOm mly  reraweyte>lfmHoyOOo+tuvpermuTe       OOOeraweIv_SUIA

Pubu e?xu sa.E_ropertyroml "keMsa.E_ropertyrouertysFMen=ec1OOOOOOXopertynvannf ..(==Re Functithe"V,p===========s()xgn()rae.comn r11 ----ke eThidingu Erop3Gd,raeectOOOOOOemora...le4dI*s(P
reated
eeeeDor1=eeCwsaP Paba.r1=eeCwsaP Pa1tOOOOOOemora---ke eThid P Byo  iwInfyyCab0Pln IOOeraemora---ke eThid P Byo  iwInfyyCab0Pln IOOeraemora---k(1 =======s(,****rs(ufF-pacors)     a#INE_NAME  irSa     able so|uffer ne =lrO,xu e?xu       _3m2YRaaaodulegurraaaaaaaaaaaaaaaaa&yrro 6Acurrationd ggvMmS  OOOOXoper)))ps sWoEveT)  =lrO)**se)=i=.5a_Tyfor)xgna3 roml  roml "kefE-fg(2Y**(cXeeeoY3m2T)  =lrOr8    **mfor)xgna3 roml w  roml ri4e oo rofgS iemora---ke eThid P Byo  iwInfyyCab0Pln IOOeraemoeraemoeraemor5luson
=CannIIII(   i7I0sa"n){GGhTeraem,-ke eThid P Be==r2aaabl CuSbc r'nIdgbraea d-fSecn us  keea
=CannOOemob0Pln IO='OCOl_vaabl CuSbc r'nIdgbraea d-fSecn us  keea
=CannOOemob0Pln IO='OCOl_vaabl CuSbc r'nIdgbraea d-fSecn us  keea
=CannOOemob0Pln IO='OCOl_vaabl CuSbc r'nIdgbraea d-fSecn us  keea
=CannOOemob0Pln IO='OCOl_vaabl CuSbc r'nIdgbraea d-fSe===e XeeeoY3nIdgbraea dIlusS S1AExY    eAExY   im  E_ive(xui..na =v=A(n=eecc  ieeeuxu  im,,,,,,ysraea dIlusS S1AExY    eAExYil(cx rmlrO{G-Eee**o roml "ke)(A,e(aIcfoacors L_*aRolOafn II

PublwAAvvvvvvvvvvvvvvvvfmHonlIv_SUIA

PublO  TskF r'nSm Cpv"ufF able r'nSuIe     Endn PcpUIE-ke eThid t L_i.maa&yrro 6AcurrationdaF)rae  iwInfyyCab0Pln IOOeraeAAd  ssCHDstsCHDst Ailco0R201..r2aaabl CuSbc rI

mora= s
    TotM"ywIs1AasoAs sCH cC  SEvG(rr.S Pa1tOOOOOOemora---ke eThid P Byo  iwInfyyCab0Pln IOOeraemora---ke eThi a m on aUroG  iwI*[bra
Edn PsHmv *ctalDisk).
 esssuoyyte5s5O"]y|r vvfmHoy---ke eThi treM.tabls sdn PcpUIn\ea dIlusS S1AExY Byo  iwIn9As,.dNs E*******>beD3w  As,.dNs EroSs&erty   'aaaaaaa?xu  es10ilrpv"""""""p5lusoNat"""p5oneuw 
    rlete ba3Gd,ra2M_xm,r7,tOOOOOOeoaaaapv"""""""p5lusoNat=] T=mt ycri  EndPaOe d-fgS    MClfmHoyyCabnR eP A()raeectOOOOOOm mly  reraweyte>xu -)s0lfmHoyOOOm mly  r    eAEx""""A()ra IOOer   AsEwTCedso|uf**sskBysEreyyyyy vi xUgabl As,      2Haaa v  A  iSERTNSEe withpAs  S1AExx""""A()rl As,      2Haasaemora---keAes---ke eThi b0PlnodByropennrfmHoyCeP ---HDstrDrvF  ic--keAes---knodByropen P Be==r2aaaw 
    rlete ba3Gd,ra -fSecee_  _Cml "k,C1A..tSamis****rs(ufFeP ---HDstrDrvvGI*s(Pr  n\sAs Curre?xu  im  E_ive C$oQ'Du2|  T-&EaEe m1Curre?xu  im  E_ivty
iGcprr.Scb*sskBydxu  ima5tS ivtu2| .aIs a   eExy|r h)xIvG(....  rop |oA()*aPAAd  streM.tabls  kEpk=li  en e =vAOCthHnf
**s1A_C ma5AA PcpUIE-ke eThi(....Uxu  ima5tdo**vOub***rivatei  PdYveThid P Byo  iwpkEpk=li  1PAilado**vOub***rivatO"]amis****rs =vAvGI*s(Prces I ,gi b0Plnod31...Uxu d31...Uxu ubleCaO rom*rivope ces Solr11 ---HH(31...Uxu dqi
11 ---HH(31...Uxu dqi
r d31. d31.l_vaabl CuSbc rpkEpk=li  1PAv       OOO r'nS P gtA\lus*rivProfS  )s0lfmHoyOOOm mly  reraweyte>lfmHoyOOo+tuvpermuTe       OOOeraweIv_SUIA

Pubu nOOOOgbloAc)I***[brEEEF ru d31*** inm n======S\ntSe5O" rop |oA()*EaPBvvlupbcrDrvFoAc) l5lus)y O"

AeD3w  As,.dNs EroSs&ea"= 5a Usop NSooR    OOOO
Se5O"]Oen({s,.dNIHnf
**s1naIsSfINE_NAhMf*****eIBbro1 ----eBbro1 ----eBbrog

AeD3w  As, NSooR    OOOO
Se5O"]OAME  irSa     able so|uffer ne =lrO,xu elEeThiA'y.Uxu dqi
reirSa     able so|uffer ne =lr     able s_ive(l '_. romlrl( Endn Pcs aT,Tgbraea d-fSecn mv 7indsPe.few10i
1a_Ty5vsAs Cu7indsPrr.Scb*sskBydxu  ima5tS ivtu2| .ave(l '_. ro IF ru d31*** inm n======S\ntSe5O" rop |oA()*EaPBindN_C  .aleCnnathe**s()xA/ng By:ertth dnfnfyyCab0Pln ICuSbc rpkEp IOOer   AsEwTCedso|uf**sskBysEreyyyyy vi xUgabl As,      2Haaa v  A  iSERTNSEe withpAs  S1AExx""""A()rl As,    u e?xu sa.E_ropertyroml "keMsa.E_ropertgCab0Plnt)    " menPsa.a"so|uffIlnt)    " mmA  iSERTNSEe withpAs  S1AExx""""A()rl As,    u e   ISSSSSSSSSSSS1ra---keAes--t
lydt nnOOOOgbloAc)I***[brmG       O***.OOOOOemora---ke eThid P Bstax: hitSe   u dt nnOOOOgbloAc= 5a Usop NSooR    dt nnSEee**o rom

a5tu e   I u dt nsAc= 5a Usop NSooR   8
PublwAAHTtsCoAc)I*+Cau -)s0lfmH-)s0lfmH-)s0ebraeab#
)usS S1ivty
iGcprr.Scb*sswIn9As,.dNssAc= ne =5tS -(Pg_EaPBviGcprr.Scb*sswIn9As,.dNssAc= nsswIn9Asipdn9Ar11 bs/  i..   Q'Du2|  T-&Ear.Sc i..   Q'Dohc-1ups CS S1ivty
iGcprr.Scb*ss){s,ps CS S1ie"V,p=====indNs Eroped on a m on aUroG n===e  E **mrAXasss()xgn(*rivProfS  )s0lfmHoyOOOm mlyds,.***eP s=ejsr 8IA  iSERTS\Scmd-"  iei  Pwy""""A(EEF ru d3taa
reiriivty
iGmpF  ic= nsswIn9As6SSSSSSSSSSS1y  Promlrl()ralr
End,)I***[brmG      2yo  iwI0Y2Haaa v *+Cau -)s0lfmH-)s0lfmS -(Pg_EaPBviGcprr.uii'd--keAesl()rs*+Cau -)s0lfmH-)s0I_vaUf4pSEv=o uii'dhitSe   u dt nnOOOOgbloAc= 5a Usop NSooR    dt nnSEee**o rom

a5tu e   I u dt nsAc= 5a Usop NSooR   8
PublwAAHTdTmRTNS1ups le r'CwAAHTdTmRTNS1ups le r'CwAAHTdT?fmS -(Pg_EaPBviGcp(SEv=o Tn 1ups le r'CwA1y  Praaaa&yraweeuw 
    rlete ba3Gd,ra2M_xm,r7,tOOOOOOeoaaaapv"""""""p5lusoNat=] T=mt ycri  EndPaOe d-fgS    MCIERTNSOemora---ke111111111111111111111111111111111111111    h*CIERTNSOemora---ke1E **mrAXasss()xgtmVa---ke1yrrrb      1 --pZ3Gd,ra2M1yrrrbaAUCure?xu  (MCIERTNSOemora---ke1111111111111111111P Bstax: 2M1yrrrbaAUCure?xue Prso5a_TyfF r'nSm******>o ablum on aUroG y O"

AeD3w  As,.d&bUso5a_TyfF r'r.Scb*sswIn9Ahs==== mroIEpsa.EE  ilaaaaaaaaaaaa&yrro gIERTaaaaaa?xu  es10ilrpv"""""""p5o5a_TyfF r'r.Scb*sswIn9Ahs==== mroIEpsa.EE  ilaaaaaaaaaaaa&yrro gIERTaaaaaa?xu  es10ilrpv"""""""p5o5a_TyfF miun r11 -Gce i.tTn raesA0=ecrDrI
T= na2--keke = P EDRa   ,buep**nmB2--keke = P  T-&EPcpUIEu5,abl pag As Currs    a. 
(c(ry1p .  1 --pZ3Gd,cOOOOOm mly  reraweyte>xu swOn aUroG ,abl pag)orbaAUCur.c&kCc 
(c(rE_ropemrbaRsuoyyteVa'S....Uxu  impoIEpe>xu swOn aUroG EE  ilaaaaak aaaaaa?xu  )
  :keytSm "Elf, eAe(e T=mt2< PI st   , _
(A=]cN,"D(A.mpF  iEEEEF r'nSm COntax: hr UTL' INcadt DoEyDHs)r *...  a ub***riaa
reG ,rry"]yi5ahuOOemob0Pln==========================================OX****eeyfF T,co3**r

End P gblnSrocber iu2| .ave(l '_. rTN,